from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


'''
Ensuring that the pyxform_test_case.PyxformTestCase class does some
internal conversions correctly.
'''


class PyxformTestCaseNonMarkdownSurveyAlternatives(PyxformTestCase):
    def test_tainted_vanilla_survey_failure(self):
        """
        the _invalid_ss_structure structure should fail to compile
        because the note has no label.

        if "errored" parameter is not set to False, it should
        raise an exception
        """
        _invalid_ss_structure = {'survey': [{'type': 'note', 'name': 'n1'}]}

        def _no_valid_flag():
            """
            when the 'errored' flag is set to false (default) and the survey
            fails to compile, the test should raise an exception.
            """
            self.assertPyxformXform(
                ss_structure=_invalid_ss_structure,
                errored=False,  # errored=False by default
            )

        self.assertRaises(Exception, _no_valid_flag)

        # however when errored=True is present,
        self.assertPyxformXform(
            ss_structure=_invalid_ss_structure,
            errored=True,
            error__contains=[
                "The survey element named 'n1' has no label or hint.",
            ],
        )

    def test_vanilla_survey(self):
        """
        testing that a survey can be passed as a _spreadsheet structure_ named
        'ss_structure'.

        this will be helpful when testing whitespace constraints and
        cell data types since markdown'd surveys strip spaces and
        cast empty strings to None values
        """
        self.assertPyxformXform(
            ss_structure={
                'survey': [{'type': 'note', 'name': 'n1', 'label': 'Note 1'}]
            },
            errored=False,
        )


class XlsFormPyxformSurveyTest(PyxformTestCase):
    def test_formid_is_not_none(self):
        """
        When the form id is not set, it should never use python's
        None. Fixing because this messes up other tests.
        """
        s1 = self.md_to_pyxform_survey("""
            | survey |      |      |       |
            |        | type | name | label |
            |        | note | q    | Q     |
            """, {}, autoname=True)

        if s1.id_string in ['None', None]:
            self.assertRaises(Exception, lambda: s1.validate())



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


MD = '''
| survey   |                                                                                              |      |       |
|          | type                                                                                         | name | label |
|          | note                                                                                         | q    | Q     |
| settings |                                                                                              |      |       |
|          | namespaces                                                                                   |      |       |
|          | esri="http://esri.com/xforms" enk="http://enketo.org/xforms" naf="http://nafundi.com/xforms" |      |       |
'''  # nopep8


class CustomXMLNamespacesTest(PyxformTestCase):
    def test_custom_xml_name_spaces(self):
        # re: https://github.com/XLSForm/pyxform/issues/65
        self.assertPyxformXform(
            name="custom_namespaces",
            md=MD,
            xml__contains=[
                'xmlns="http://www.w3.org/2002/xforms"',
                'xmlns:h="http://www.w3.org/1999/xhtml"',
                'xmlns:jr="http://openrosa.org/javarosa"',
                'xmlns:orx="http://openrosa.org/xforms"',
                'xmlns:xsd="http://www.w3.org/2001/XMLSchema"',
                'xmlns:esri="http://esri.com/xforms"',
                'xmlns:enk="http://enketo.org/xforms"',
                'xmlns:naf="http://nafundi.com/xforms"',
            ],
        )



import unittest2 as unittest
import codecs
import os
import pyxform
from pyxform.utils import sheet_to_csv
from pyxform.tests.utils import XFormTestCase

DIR = os.path.dirname(__file__)


class MainTest(XFormTestCase):
    
    maxDiff = None
    
    def runTest(self):
        for filename in ["select_one_external.xlsx"]:
            path_to_excel_file = os.path.join(DIR, "example_xls", filename)
            # Get the xform output path:
            root_filename, ext = os.path.splitext(filename)
            output_path = os.path.join(
                DIR, "test_output", root_filename + ".xml")
            expected_output_path = os.path.join(
                DIR, "test_expected_output", root_filename + ".xml")
            output_csv = os.path.join(
                DIR, "test_output", root_filename + ".csv")
            # Do the conversion:
            json_survey = pyxform.xls2json.parse_file_to_json(
                path_to_excel_file)

            self.assertTrue(sheet_to_csv(
                path_to_excel_file, output_csv, "external_choices"))
            self.assertFalse(sheet_to_csv(
                path_to_excel_file, output_csv, "non-existant sheet"))

            survey = pyxform.create_survey_element_from_dict(json_survey)

            survey.print_xform_to_file(output_path)

            # Compare with the expected output:
            with codecs.open(expected_output_path, 'rb', encoding="utf-8") as\
                    expected_file:
                with codecs.open(output_path, 'rb', encoding="utf-8") as \
                        actual_file:
                    self.assertXFormEqual(
                        expected_file.read(), actual_file.read())
                
if __name__ == '__main__':
    unittest.main()



"""
Testing creation of Surveys using verbose methods
"""
from unittest import TestCase

from pyxform import *
from pyxform.builder import create_survey_element_from_dict
from pyxform.tests.utils import prep_class_config

TESTING_BINDINGS = True


def ctw(control):
    """
    ctw stands for control_test_wrap, but ctw is shorter and easier. using
    begin_str and end_str to take out the wrap that xml gives us
    """
    return control.toxml()


class Json2XformQuestionValidationTests(TestCase):
    maxDiff = None

    @classmethod
    def setUpClass(cls):
        prep_class_config(cls=cls)

    def setUp(self):
        self.s = Survey(name=u"test")

    def test_question_type_string(self):
        simple_string_json = {
            u"label": {
                u"French": u"Nom du travailleur agricole:",
                u"English": u"Name of Community Agricultural Worker"
            },
            u"type": u"text",
            u"name": u"enumerator_name"
        }

        q = create_survey_element_from_dict(simple_string_json)

        expected_string_control_xml = self.config.get(
            self.cls_name, "test_question_type_string_control")

        expected_string_binding_xml = self.config.get(
            self.cls_name, "test_question_type_string_binding")

        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()), expected_string_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()), expected_string_binding_xml)

    def test_select_one_question_multilingual(self):
        """
        Test the lowest common denominator of question types.
        """
        simple_select_one_json = {
            u"label": {u"f": u"ftext", u"e": u"etext"},
            u"type": u"select one",
            u"name": u"qname",
            u"choices": [
                {u"label": {u"f": u"fa", u"e": u"ea"}, u"name": u"a"},
                {u"label": {u"f": u"fb", u"e": u"eb"}, u"name": u"b"}
            ]
        }

        # I copied the response in, since this is not our method of testing
        # valid return values.
        expected_select_one_control_xml = self.config.get(
            self.cls_name, "test_select_one_question_multilingual_control")

        expected_select_one_binding_xml = self.config.get(
            self.cls_name, "test_select_one_question_multilingual_binding")

        q = create_survey_element_from_dict(simple_select_one_json)
        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()), expected_select_one_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()),
                             expected_select_one_binding_xml)

    def test_simple_integer_question_type_multilingual(self):
        """
        not sure how integer questions should show up.
        """
        simple_integer_question = {
            u"label": {u"f": u"fc", u"e": u"ec"},
            u"type": u"integer",
            u"name": u"integer_q",
            u"attributes": {}
        }

        expected_integer_control_xml = self.config.get(
            self.cls_name,
            "test_simple_integer_question_type_multilingual_control")

        expected_integer_binding_xml = self.config.get(
            self.cls_name,
            "test_simple_integer_question_type_multilingual_binding")

        q = create_survey_element_from_dict(simple_integer_question)

        self.s.add_child(q)

        self.assertEqual(ctw(q.xml_control()), expected_integer_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()), expected_integer_binding_xml)

    def test_simple_date_question_type_multilingual(self):
        """
        not sure how date questions should show up.
        """
        simple_date_question = {u"label": {u"f": u"fd", u"e": u"ed"},
                                u"type": u"date", u"name": u"date_q",
                                u"attributes": {}}

        expected_date_control_xml = self.config.get(
            self.cls_name,
            "test_simple_date_question_type_multilingual_control")

        expected_date_binding_xml = self.config.get(
            self.cls_name,
            "test_simple_date_question_type_multilingual_binding")

        q = create_survey_element_from_dict(simple_date_question)
        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()), expected_date_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()), expected_date_binding_xml)

    def test_simple_phone_number_question_type_multilingual(self):
        """
        not sure how phone number questions should show up.
        """
        simple_phone_number_question = {
            u"label": {u"f": u"fe", u"e": u"ee"},
            u"type": u"phone number",
            u"name": u"phone_number_q",
        }

        expected_phone_number_control_xml = self.config.get(
            self.cls_name,
            "test_simple_phone_number_question_type_multilingual_control")

        expected_phone_number_binding_xml = self.config.get(
            self.cls_name,
            "test_simple_phone_number_question_type_multilingual_binding")

        q = create_survey_element_from_dict(simple_phone_number_question)
        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()),
                         expected_phone_number_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()),
                             expected_phone_number_binding_xml)

    def test_simple_select_all_question_multilingual(self):
        """
        not sure how select all questions should show up...
        """
        simple_select_all_question = {
            u"label": {u"f": u"f choisit", u"e": u"e choose"},
            u"type": u"select all that apply",
            u"name": u"select_all_q",
            u"choices": [
                {u"label": {u"f": u"ff", u"e": u"ef"}, u"name": u"f"},
                {u"label": {u"f": u"fg", u"e": u"eg"}, u"name": u"g"},
                {u"label": {u"f": u"fh", u"e": u"eh"}, u"name": u"h"}
            ]
        }

        expected_select_all_control_xml = self.config.get(
            self.cls_name,
            "test_simple_select_all_question_multilingual_control")

        expected_select_all_binding_xml = self.config.get(
            self.cls_name,
            "test_simple_select_all_question_multilingual_binding")

        q = create_survey_element_from_dict(simple_select_all_question)
        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()), expected_select_all_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()),
                             expected_select_all_binding_xml)

    def test_simple_decimal_question_multilingual(self):
        """
        not sure how decimal should show up.
        """
        simple_decimal_question = {u"label": {u"f": u"f text", u"e": u"e text"},
                                   u"type": u"decimal", u"name": u"decimal_q",
                                   u"attributes": {}}

        expected_decimal_control_xml = self.config.get(
            self.cls_name, "test_simple_decimal_question_multilingual_control")

        expected_decimal_binding_xml = self.config.get(
            self.cls_name, "test_simple_decimal_question_multilingual_binding")

        q = create_survey_element_from_dict(simple_decimal_question)
        self.s.add_child(q)
        self.assertEqual(ctw(q.xml_control()), expected_decimal_control_xml)

        if TESTING_BINDINGS:
            self.assertEqual(ctw(q.xml_binding()), expected_decimal_binding_xml)



from unittest import TestCase
from pyxform.builder import create_survey_from_path
from pyxform.tests import utils


class TutorialTests(TestCase):
    def test_create_from_path(self):
        path = utils.path_to_text_fixture("tutorial.xls")
        survey = create_survey_from_path(path)
        # print survey.to_xml()



from pyxform.question import SurveyElement
from pyxform.utils import node
from pyxform.errors import PyXFormError


class Section(SurveyElement):
    def validate(self):
        super(Section, self).validate()
        for element in self.children:
            element.validate()
        self._validate_uniqueness_of_element_names()

    # there's a stronger test of this when creating the xpath
    # dictionary for a survey.
    def _validate_uniqueness_of_element_names(self):
        element_slugs = []
        for element in self.children:
            if element.name in element_slugs:
                raise PyXFormError(
                    "There are two survey elements named '%s' in the section"
                    " named '%s'." % (element.name, self.name)
                    )
            element_slugs.append(element.name)

    def xml_instance(self, **kwargs):
        """
        Creates an xml representation of the section
        """
        attributes = {}
        attributes.update(kwargs)
        attributes.update(self.get(u'instance', {}))
        survey = self.get_root()
        # Resolve field references in attributes
        for key, value in attributes.items():
            attributes[key] = survey.insert_xpaths(value)
        result = node(self.name, **attributes)
        for child in self.children:
            if child.get(u"flat"):
                for grandchild in child.xml_instance_array():
                    result.appendChild(grandchild)
            else:
                result.appendChild(child.xml_instance())
        return result

    def xml_instance_array(self):
        """
        This method is used for generating flat instances.
        """
        for child in self.children:
            if child.get(u"flat"):
                for grandchild in child.xml_instance_array():
                    yield grandchild
            else:
                yield child.xml_instance()

    def xml_control(self):
        """
        Ideally, we'll have groups up and rolling soon, but for now
        let's just yield controls from all the children of this section
        """
        for e in self.children:
            control = e.xml_control()
            if control is not None:
                yield control


class RepeatingSection(Section):
    def xml_control(self):
        """
        <group>
        <label>Fav Color</label>
        <repeat nodeset="fav-color">
          <select1 ref=".">
            <label ref="jr:itext('fav')" />
            <item><label ref="jr:itext('red')" /><value>red</value></item>
            <item><label ref="jr:itext('green')" /><value>green</value></item>
            <item><label ref="jr:itext('blue')" /><value>blue</value></item>
          </select1>
        </repeat>
        </group>
        """
        control_dict = self.control.copy()
        survey = self.get_root()
        # Resolve field references in attributes
        for key, value in control_dict.items():
            control_dict[key] = survey.insert_xpaths(value)
        repeat_node = node(u"repeat", nodeset=self.get_xpath(), **control_dict)

        for n in Section.xml_control(self):
            repeat_node.appendChild(n)

        label = self.xml_label()
        if label:
            return node(
                u"group", self.xml_label(), repeat_node,
                ref=self.get_xpath()
                )
        return node(u"group", repeat_node, ref=self.get_xpath(),
                    **self.control)

    # I'm anal about matching function signatures when overriding a function,
    # but there's no reason for kwargs to be an argument
    def xml_instance(self, **kwargs):
        kwargs = {"jr:template": ""}  # It might make more sense to add this
        #                               as a child on initialization

        return super(RepeatingSection, self).xml_instance(**kwargs)


class GroupedSection(Section):
    # I think this might be a better place for the table-list stuff, however it
    # doesn't allow for as good of validation as putting it in xls2json
    # def __init__(self, **kwargs):
    #        control = kwargs.get(u"control")
    #        if control:
    #            appearance = control.get(u"appearance")
    #            if appearance is u"table-list":
    #                print "HI"
    #                control[u"appearance"] = "field-list"
    #                kwargs["children"].insert(0, kwargs["children"][0])
    #        super(GroupedSection, self).__init__(kwargs)

    def xml_control(self):
        control_dict = self.control

        if control_dict.get("bodyless"):
            return None

        children = []
        attributes = {}
        attributes.update(self.control)

        survey = self.get_root()

        # Resolve field references in attributes
        for key, value in attributes.items():
            attributes[key] = survey.insert_xpaths(value)

        if not self.get('flat'):
            attributes['ref'] = self.get_xpath()

        if 'label' in self and len(self['label']) > 0:
            children.append(self.xml_label())
        for n in Section.xml_control(self):
            children.append(n)

        if u"appearance" in control_dict:
            attributes['appearance'] = control_dict['appearance']

        if u"intent" in control_dict:
            survey = self.get_root()
            attributes['intent'] = survey.insert_xpaths(control_dict['intent'])

        return node(u"group", *children, **attributes)

    def to_json_dict(self):
        # This is quite hacky, might want to think about a smart way
        # to approach this problem.
        result = super(GroupedSection, self).to_json_dict()
        result[u"type"] = u"group"
        return result



from pyxform import create_survey_from_xls
import re
from pyxform.tests import utils
from unittest2 import TestCase
from xml.dom.minidom import getDOMImplementation
from pyxform.utils import node


class XMLTests(utils.XFormTestCase):
    def setUp(self):
        self.survey = create_survey_from_xls(
            utils.path_to_text_fixture("yes_or_no_question.xls"))

    def test_to_xml(self):
        xml_str = u'''<?xml version="1.0"?>
<h:html xmlns="http://www.w3.org/2002/xforms"
    xmlns:ev="http://www.w3.org/2001/xml-events"
    xmlns:h="http://www.w3.org/1999/xhtml"
    xmlns:jr="http://openrosa.org/javarosa"
    xmlns:orx="http://openrosa.org/xforms"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <h:head>
    <h:title>yes_or_no_question</h:title>
    <model>
      <itext>
        <translation lang="english">
          <text id="/yes_or_no_question/good_day:label">
            <value>have you had a good day today?</value>
          </text>
          <text id="/yes_or_no_question/good_day/no:label">
            <value>no</value>
          </text>
          <text id="/yes_or_no_question/good_day/yes:label">
            <value>yes</value>
          </text>
        </translation>
      </itext>
      <instance>
        <yes_or_no_question id="yes_or_no_question_2011_04_22">
          <good_day/>
          <meta>
            <instanceID/>
          </meta>
        </yes_or_no_question>
      </instance>
      <bind nodeset="/yes_or_no_question/good_day" type="select1"/>
      <bind calculate="concat('uuid:', uuid())"
        nodeset="/yes_or_no_question/meta/instanceID"
        readonly="true()" type="string"/>
    </model>
  </h:head>
  <h:body>
    <select1 ref="/yes_or_no_question/good_day">
      <label ref="jr:itext('/yes_or_no_question/good_day:label')"/>
      <item>
        <label ref="jr:itext('/yes_or_no_question/good_day/yes:label')"/>
        <value>yes</value>
      </item>
      <item>
        <label ref="jr:itext('/yes_or_no_question/good_day/no:label')"/>
        <value>no</value>
      </item>
    </select1>
  </h:body>
</h:html>
'''
        xml_str = re.sub(r"yes_or_no_question_2011_04_22",
                         self.survey.id_string, xml_str)
        self.maxDiff = None
        self.assertXFormEqual(xml_str, self.survey.to_xml())


class MinidomTextWriterMonkeyPatchTest(TestCase):

    def test_patch_lets_node_func_escape_only_necessary(self):
        """Should only escape text chars that should be: ["<", ">", "&"]."""
        text = u"' \" & < >"
        expected = u"<root>' \" &amp; &lt; &gt;</root>".format(text)
        observed = node(u"root", text).toprettyxml(indent="", newl="")
        self.assertEqual(expected, observed)

    def test_original_escape_escapes_more_than_necessary(self):
        """Should fail if the original is updated (the patch can be removed)."""
        text = u"' \" & < >"
        expected = u"<root>' &quot; &amp; &lt; &gt;</root>".format(text)
        document = getDOMImplementation().createDocument(None, u"root", None)
        root = document.documentElement
        text_node = document.createTextNode(text)
        root.appendChild(text_node)
        observed = root.toprettyxml(indent="", newl="")
        self.assertEqual(expected, observed)



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class GroupsTests(PyxformTestCase):

    def test_group_type(self):
        self.assertPyxformXform(
            md="""
            | survey |             |         |                  |
            |        | type        | name    | label            |
            |        | text        | pregrp  | Pregroup text    |
            |        | begin group | xgrp    | XGroup questions |
            |        | text        | xgrp_q1 | XGroup Q1        |
            |        | integer     | xgrp_q2 | XGroup Q2        |
            |        | end group   |         |                  |
            |        | note        | postgrp | Post group note  |
            """,
            model__contains=[
                '<pregrp/>',
                '<xgrp>',
                    '<xgrp_q1/>',  # nopep8
                    '<xgrp_q1/>',  # nopep8
                    '<xgrp_q2/>',  # nopep8
                '</xgrp>',
                '<postgrp/>',
                ])

    def test_group_intent(self):
        self.assertPyxformXform(
            name="intent_test",
            md="""
            | survey |             |         |                  |                                                             |
            |        | type        | name    | label            | intent                                                      |
            |        | text        | pregrp  | Pregroup text    |                                                             |
            |        | begin group | xgrp    | XGroup questions | ex:org.redcross.openmapkit.action.QUERY(osm_file=${pregrp}) |
            |        | text        | xgrp_q1 | XGroup Q1        |                                                             |
            |        | integer     | xgrp_q2 | XGroup Q2        |                                                             |
            |        | end group   |         |                  |                                                             |
            |        | note        | postgrp | Post group note  |                                                             |
            """,  # nopep8
            xml__contains=[
                '<group intent="ex:org.redcross.openmapkit.action.QUERY(osm_file= /intent_test/pregrp )" ref="/intent_test/xgrp">',  # nopep8
                ])



"""
Testing creation of Surveys using verbose methods
"""
from unittest import TestCase
from pyxform import *
from pyxform.tests import utils


class Json2XformVerboseSurveyCreationTests(TestCase):
    def test_survey_can_be_created_in_a_verbose_manner(self):
        s = Survey()
        s.name = "simple_survey"

        q = MultipleChoiceQuestion()
        q.name = "cow_color"
        q.type = "select one"

        q.add_choice(label="Green", name="green")
        s.add_child(q)

        expected_dict = {
            u'name': 'simple_survey',
            u'children': [
                {
                    u'name': 'cow_color',
                    u'type': 'select one',
                    u'children': [
                        {
                            u'label': 'Green',
                            u'name': 'green',
                        }
                    ],
                }
            ],
        }
        self.maxDiff = None
        self.assertEqual(s.to_json_dict(), expected_dict)

    def test_survey_can_be_created_in_a_slightly_less_verbose_manner(self):
        option_dict_array = [
            {'name': 'red', 'label': 'Red'},
            {'name': 'blue', 'label': 'Blue'}
        ]

        q = MultipleChoiceQuestion(name="Favorite_Color",
                                   choices=option_dict_array)
        q.type = u"select one"
        s = Survey(name="Roses_are_Red", children=[q])

        expected_dict = {
            u'name': 'Roses_are_Red',
            u'children': [
                {
                    u'name': 'Favorite_Color',
                    u'type': u'select one',
                    u'children': [
                        {u'label': 'Red', u'name': 'red'},
                        {u'label': 'Blue', u'name': 'blue'}
                    ],
                }
            ],
        }

        self.assertEqual(s.to_json_dict(), expected_dict)

    def test_two_options_cannot_have_the_same_value(self):
        q = MultipleChoiceQuestion(name="Favorite Color")
        q.add_choice(name="grey", label="Gray")
        q.add_choice(name="grey", label="Grey")
        self.assertRaises(Exception, q, 'validate')

    def test_one_section_cannot_have_two_conflicting_slugs(self):
        q1 = InputQuestion(name="YourName")
        q2 = InputQuestion(name="YourName")
        s = Survey(name="Roses are Red", children=[q1, q2])
        self.assertRaises(Exception, s, 'validate')

    def allow_surveys_with_comment_rows(self):
        """assume that a survey with rows that don't have name, type, or label
        headings raise warning only"""
        path = utils.path_to_text_fixture('allow_comment_rows_test.xls')
        survey = create_survey_from_xls(path)
        expected_dict = {
            "default_language": "default",
            "id_string": "allow_comment_rows_test",
            "children": [
                {
                    "name": "farmer_name",
                    "label": {
                        "English": "First and last name of farmer"
                    },
                    "type": "text"
                }
            ],
            "name": "allow_comment_rows_test",
            "_translations": {
                "English": {
                    "/allow_comment_rows_test/farmer_name:label": {
                        "long": "First and last name of farmer"
                    }
                }
            },
            "title": "allow_comment_rows_test",
            "_xpath": {
                "allow_comment_rows_test": "/allow_comment_rows_test",
                "farmer_name": "/allow_comment_rows_test/farmer_name"
            },
            "type": "survey"
        }
        self.assertEquals(survey.to_json_dict(), expected_dict)



from unittest import TestCase
from pyxform.builder import create_survey_from_path
import os
from pyxform.tests import utils


class DumpAndLoadTests(TestCase):

    def setUp(self):
        self.excel_files = [
            "gps.xls",
            # "include.xls",
            "specify_other.xls",
            "group.xls",
            "loop.xls",
            "text_and_integer.xls",
            # todo: this is looking for json that is created (and
            # deleted) by another test, is should just add that json
            # to the directory.
            # "include_json.xls",
            "simple_loop.xls",
            "yes_or_no_question.xls",
            ]
        self.surveys = {}
        self.this_directory = os.path.dirname(__file__)
        for filename in self.excel_files:
            path = utils.path_to_text_fixture(filename)
            try:
                self.surveys[filename] = create_survey_from_path(path)
            except Exception as e:
                print("Error on : " + filename)
                raise e

    def test_load_from_dump(self):
        for filename, survey in self.surveys.items():
            survey.json_dump()
            path = survey.name + ".json"
            survey_from_dump = create_survey_from_path(path)
            self.assertEqual(survey.to_json_dict(),
                             survey_from_dump.to_json_dict())

    def tearDown(self):
        for filename, survey in self.surveys.items():
            path = survey.name + ".json"
            os.remove(path)



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestWarnings(PyxformTestCase):
    def test_l1(self):
        self.assertPyxformXform(
            name="test_l1",
            md="""
                | survey |      |           |        |
                |        | type | name      | hint   |
                |        | text | some_text | a hint |
                """,
            instance__contains=[
                '<some_text/>',
                ],
            model__contains=[
                '<bind nodeset="/test_l1/some_text" type="string"/>',
                ],
            xml__contains=[
                '<input ref="/test_l1/some_text">',
                  '<hint>a hint</hint>',  # nopep8
                '</input>',
            ],
        )

    def test_l2(self):
        self.assertPyxformXform(
            name="img_test",
            md="""
                | survey |      |                  |              |
                |        | type | name             | image        |
                |        | note | display_img_test | img_test.jpg |
                """,
            model__contains=[
                '<bind nodeset="/img_test/display_img_test" readonly="true()" type="string"/>',  # nopep8

                ],
            instance__contains=[
                '<display_img_test/>',
                ],
            xml__contains=[
                '<translation default="true()" lang="default">',
                # and further down...
                """<label ref="jr:itext('/img_test/display_img_test:label')"/>"""  # nopep8

            ],
        )



from unittest import TestCase
from pyxform.builder import create_survey_from_path
from pyxform.xls2json import SurveyReader
from pyxform.tests import utils


class SettingsTests(TestCase):

    maxDiff = None

    def setUp(self):
        self.path = utils.path_to_text_fixture("settings.xls")

    def test_survey_reader(self):
        survey_reader = SurveyReader(self.path)
        expected_dict = {
            u'id_string': u'new_id',
            u'sms_keyword': u'new_id',
            u'default_language': u'default',
            u'name': u'settings',
            u'title': u'My Survey',
            u'type': u'survey',
            u'attribute': {
                u'my_number': u'1234567890',
                u'my_string': u'lor\xe9m ipsum'
            },
            u'children': [
                {
                    u'name': u'your_name',
                    u'label': {u'english': u'What is your name?'},
                    u'type': u'text'
                },
                {
                    u'name': u'your_age',
                    u'label': {u'english': u'How many years old are you?'},
                    u'type': u'integer'
                },
                {
                    'children': [
                        {
                            'bind': {
                                'calculate': "concat('uuid:', uuid())",
                                'readonly': 'true()'
                            },
                            'name': 'instanceID',
                            'type': 'calculate'
                        }
                    ],
                    'control': {
                        'bodyless': True
                    },
                    'name': 'meta',
                    'type': 'group'
                }
            ],
        }
        self.assertEqual(survey_reader.to_json_dict(), expected_dict)

    def test_settings(self):
        survey = create_survey_from_path(self.path)
        self.assertEqual(survey.id_string, "new_id")
        self.assertEqual(survey.title, "My Survey")



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class RoundCalculationTest(PyxformTestCase):
    def test_non_existent_itext_reference(self):
        self.assertPyxformXform(
            name="ecsv",
            md="""
            | survey |             |                |         |                     |
            |        | type        | name           | label   | calculation         |
            |        | decimal     | amount         | Counter |                     |
            |        | calculate   | rounded        | Rounded | round(${amount}, 0) |
            """,  # noqa
            xml__contains=[
                """<instance>"""
            ],
            run_odk_validate=True)



"""
Tests by file. Runs through a list of *.xls files, and expects that the output
for a *.xml with a matching prefix before the . is as expected. Possibly risky:
all tests in this file are defined according to matching files.
"""

import xml.etree.ElementTree as ETree
from formencode.doctest_xml_compare import xml_compare
from unittest import TestCase
import pyxform
from pyxform import xls2json
import os
from pyxform.tests import utils
import codecs
import sys


class MainTest(TestCase):
    
    def runTest(self):
        files_to_test = ["instance_xmlns_test.xls"]
        for file_to_test in files_to_test:
            path_to_excel_file = utils.path_to_text_fixture(file_to_test)
            
            # Get the xform output path:
            directory, filename = os.path.split(path_to_excel_file)
            root_filename, ext = os.path.splitext(filename)
            path_to_output_xform = os.path.join(
                directory, root_filename + "_output.xml")
            path_to_expected_xform = os.path.join(
                directory, root_filename + ".xml")

            # Do the conversion:
            json_survey = xls2json.parse_file_to_json(path_to_excel_file)
            survey = pyxform.create_survey_element_from_dict(json_survey)
            survey.print_xform_to_file(path_to_output_xform)
            
            # Compare with the expected output:
            with codecs.open(path_to_expected_xform, 'rb', encoding="utf-8"
                             ) as expected_file:
                expected = ETree.fromstring(expected_file.read())
                result = ETree.fromstring(survey.to_xml())

                def write_line(x): sys.stdout.write(x + "\n")
                reporter = write_line
                self.assertTrue(xml_compare(
                    expected, result, reporter=reporter))
            os.remove(path_to_output_xform)



import os
from pyxform import file_utils
from pyxform.builder import create_survey, create_survey_from_path
from unittest2 import TestCase
import xml.etree.ElementTree as ETree
from formencode.doctest_xml_compare import xml_compare
try:
    import ConfigParser
    configparser = ConfigParser
except ImportError:
    import configparser


def path_to_text_fixture(filename):
    directory = os.path.dirname(__file__)
    return os.path.join(directory, "example_xls", filename)


def build_survey(filename):
    path = path_to_text_fixture(filename)
    return create_survey_from_path(path)


def create_survey_from_fixture(fixture_name, filetype="xls",
                               include_directory=False):
    fixture_path = path_to_text_fixture("%s.%s" % (fixture_name, filetype))
    noop, section_dict = file_utils.load_file_to_dict(fixture_path)
    pkg = {u'main_section': section_dict}
    if include_directory:
        directory, noop = os.path.split(fixture_path)
        pkg[u'sections'] = file_utils.collect_compatible_files_in_directory(
            directory)
    return create_survey(**pkg)


class XFormTestCase(TestCase):
    """
    XFormTestCase provides an assertion for comparing two XForms for
    equivalence.  It relies on the xml_compare function provided by FormEncode,
    but with some extra logic to handle the essentially random order of tags in
    the <model> section.  The iteration order of dict keys in Python 3 is
    unpredictable, and many of the elements in <model> tag are generated from
    dicts.  A simple workaround is to just reorder those tags here.  An ideal
    solution might be to rewrite the code that generates an XForm to use
    OrderedDicts where appropriate.
    """

    def assertXFormEqual(self, xform1, xform2):
        xform1 = ETree.fromstring(xform1.encode('utf-8'))
        xform2 = ETree.fromstring(xform2.encode('utf-8'))

        # Sort tags under <model> section in each form
        self.sort_model(xform1)
        self.sort_model(xform2)

        # Report any errors returned from xform_compare
        errs = []

        def reporter(msg):
            errs.append(msg)

        self.assertTrue(
            xml_compare(xform1, xform2, reporter),
            "\n\n" + "\n".join(reversed(errs))
        )

    def sort_elems(self, elems, attr=None):
        if attr:
            def elem_get_attr(elem): return elem.get(attr, '')
            key = elem_get_attr
        else:
            def elem_get_tag(elem): return elem.tag
            key = elem_get_tag
        elems[:] = sorted(elems, key=key)

    def sort_model(self, xform):
        ns = "{http://www.w3.org/2002/xforms}"
        model = xform.find('.//' + ns + 'model')

        # Sort multiple <instance> tags, if any
        self.sort_elems(model, 'id')

        # Sort any <item> tags in each <instance>
        for instance in model.findall(ns + 'instance'):
            if instance.get('id', None):
                root = instance.find(ns + 'root')
                if root is not None:
                    for item in root:
                        self.sort_elems(item)

        # Sort <translation> tags as well as inner <text> and <value> tags.
        itext = model.find(ns + 'itext')
        if itext is None:
            return

        self.sort_elems(itext, 'lang')
        for translation in itext:
            self.sort_elems(translation, 'id')
            for text in translation:
                self.sort_elems(text, 'form')


def prep_class_config(cls, test_dir="tests"):
    cls.config = configparser.ConfigParser()
    here = os.path.dirname(__file__)
    root = os.path.dirname(here)
    strings = os.path.join(root, test_dir, "fixtures", "strings.ini")
    cls.config.read(strings)
    cls.cls_name = cls.__name__



# vim: ai ts=4 sts=4 et sw=4 encoding=utf-8

# todo: this has been copied from xform_manager, we need to figure out
# where this code is actually going to live.

from xml.dom import minidom
import re
from pyxform.utils import unicode

XFORM_ID_STRING = u"_xform_id_string"


def _xml_node_to_dict(node):
    assert isinstance(node, minidom.Node)
    if len(node.childNodes) == 0:
        # there's no data for this leaf node
        value = None
    elif len(node.childNodes) == 1 and \
            node.childNodes[0].nodeType == node.TEXT_NODE:
        # there is data for this leaf node
        value = node.childNodes[0].nodeValue
    else:
        # this is an internal node
        value = {}
        for child in node.childNodes:
            d = _xml_node_to_dict(child)
            child_name = child.nodeName
            assert list(d.keys()) == [child_name]
            if child_name not in value:
                # copy the value into the dict
                value[child_name] = d[child_name]
            elif type(value[child_name]) == list:
                # add to the existing list
                value[child_name].append(d[child_name])
            else:
                # create a new list
                value[child_name] = [value[child_name], d[child_name]]
    return {node.nodeName: value}


def _flatten_dict(d, prefix):
    """
    Return a list of XPath, value pairs.
    """
    assert type(d) == dict
    assert type(prefix) == list

    for key, value in d.items():
        new_prefix = prefix + [key]
        if type(value) == dict:
            for pair in _flatten_dict(value, new_prefix):
                yield pair
        elif type(value) == list:
            for i, item in enumerate(value):
                item_prefix = list(new_prefix)  # make a copy
                # note on indexing xpaths: IE5 and later has
                # implemented that [0] should be the first node, but
                # according to the W3C standard it should have been
                # [1]. I'm adding 1 to i to start at 1.
                item_prefix[-1] += u"[%s]" % unicode(i + 1)
                if type(item) == dict:
                    for pair in _flatten_dict(item, item_prefix):
                        yield pair
                else:
                    yield (item_prefix, item)
        else:
            yield (new_prefix, value)


def _get_all_attributes(node):
    """
    Go through an XML document returning all the attributes we see.
    """
    if hasattr(node, "hasAttributes") and node.hasAttributes():
        for key in node.attributes.keys():
            yield key, node.getAttribute(key)
    for child in node.childNodes:
        for pair in _get_all_attributes(child):
            yield pair


class XFormInstanceParser(object):
    def __init__(self, xml_str):
        self.parse(xml_str)

    def parse(self, xml_str):
        clean_xml_str = xml_str.strip()
        clean_xml_str = re.sub(unicode(r">\s+<"), unicode("><"), clean_xml_str)
        self._xml_obj = minidom.parseString(clean_xml_str)
        self._root_node = self._xml_obj.documentElement
        self._dict = _xml_node_to_dict(self._root_node)
        self._flat_dict = {}
        for path, value in _flatten_dict(self._dict, []):
            self._flat_dict[u"/".join(path[1:])] = value
        self._set_attributes()

    def get_root_node_name(self):
        return self._root_node.nodeName

    def get(self, abbreviated_xpath):
        return self.to_flat_dict()[abbreviated_xpath]

    def to_json_dict(self):
        return self._dict

    def to_flat_dict(self):
        return self._flat_dict

    def get_attributes(self):
        return self._attributes

    def _set_attributes(self):
        self._attributes = {}
        all_attributes = list(_get_all_attributes(self._root_node))
        for key, value in all_attributes:
            assert key not in self._attributes
            self._attributes[key] = value

    def get_xform_id_string(self):
        return self._attributes[u"id"]

    def get_flat_dict_with_attributes(self):
        result = self.to_flat_dict().copy()
        result[XFORM_ID_STRING] = self.get_xform_id_string
        return result


def xform_instance_to_dict(xml_str):
    parser = XFormInstanceParser(xml_str)
    return parser.to_json_dict()


def xform_instance_to_flat_dict(xml_str):
    parser = XFormInstanceParser(xml_str)
    return parser.to_flat_dict()


def parse_xform_instance(xml_str):
    parser = XFormInstanceParser(xml_str)
    return parser.get_flat_dict_with_attributes()



# The Django application xls2xform uses the function
# pyxform.create_survey. We have a test here to make sure no one
# breaks that function.

from unittest import TestCase
import pyxform
from pyxform.xls2xform import _create_parser


class XLS2XFormTests(TestCase):
    survey_package = {
        'id_string': u'test_2011_08_29b',
        'name_of_main_section': u'gps',
        'sections': {
            u'gps': {
                u'children': [
                    {
                        u'name': u'location',
                        u'type': u'gps'
                        }
                    ],
                u'name': u'gps',
                u'type': u'survey'
                }
            },
        'title': u'test'
        }
    survey = pyxform.create_survey(**survey_package)

    def test_create_parser_without_args(self):
        """Should exit when no args provided."""
        with self.assertRaises(SystemExit):
            _create_parser().parse_args([])

    def test_create_parser_with_args(self):
        """Should parse the provided arguments."""
        arg_xlsform = 'xlsform.xlsx'
        arg_output = '.'
        arg_list = ['--json', '--skip_validate', '--no_pretty_print',
            arg_xlsform, arg_output]
        args = _create_parser().parse_args(arg_list)
        self.assertEqual(arg_xlsform, args.path_to_XLSForm)
        self.assertEqual(arg_output, args.output_path)
        self.assertEqual(True, args.json)
        self.assertEqual(False, args.skip_validate)
        self.assertEqual(False, args.no_pretty_print)

    def test_create_parser_json_default_false(self):
        """Should have json=False if not specified."""
        arg_xlsform = 'xlsform.xlsx'
        arg_output = '.'
        arg_list = [arg_xlsform, arg_output]
        args = _create_parser().parse_args(arg_list)
        self.assertEqual(False, args.json)

    def test_create_parser_no_validate_default_true(self):
        """Should have no_validate=True if not specified."""
        arg_xlsform = 'xlsform.xlsx'
        arg_output = '.'
        arg_list = [arg_xlsform, arg_output]
        args = _create_parser().parse_args(arg_list)
        self.assertEqual(True, args.skip_validate)

    def test_create_parser_no_pretty_print_default_true(self):
        """Should have no_pretty_print=True if not specified."""
        args = _create_parser().parse_args([
            'xlsform.xlsx',
            '.',
        ])
        self.assertEqual(True, args.no_pretty_print)



from __future__ import print_function
from pyxform.builder import create_survey_from_path
from pyxform.xform2json import create_survey_element_from_xml, _try_parse
import os
from pyxform.tests import utils
from unittest2 import TestCase
from xml.etree.ElementTree import ParseError


class DumpAndLoadXForm2JsonTests(utils.XFormTestCase):

    maxDiff = None

    def setUp(self):
        self.excel_files = [
            "gps.xls",
            # "include.xls",
            "specify_other.xls",
            "loop.xls",
            "text_and_integer.xls",
            # todo: this is looking for json that is created (and
            # deleted) by another test, is should just add that json
            # to the directory.
            # "include_json.xls",
            "simple_loop.xls",
            "yes_or_no_question.xls",
            "xlsform_spec_test.xlsx",
            "group.xls",
        ]
        self.surveys = {}
        self.this_directory = os.path.dirname(__file__)
        for filename in self.excel_files:
            path = utils.path_to_text_fixture(filename)
            try:
                self.surveys[filename] = create_survey_from_path(path)
            except Exception as e:
                print("Error on : " + filename)
                raise e

    def test_load_from_dump(self):
        for filename, survey in iter(self.surveys.items()):
            survey.json_dump()
            survey_from_dump = create_survey_element_from_xml(survey.to_xml())
            self.assertXFormEqual(
                survey.to_xml(), survey_from_dump.to_xml())

    def tearDown(self):
        for filename, survey in self.surveys.items():
            path = survey.name + ".json"
            if os.path.exists(path):
                os.remove(path)


class TestXMLParse(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.cwd = os.path.dirname(__file__)
        cls.tidy_file = None
        cls.xml = """<?xml version="1.0"?>\n<a><b>1</b></a>"""

    def tearDown(self):
        if self.tidy_file is not None:
            os.remove(self.tidy_file)

    def test_try_parse_with_string(self):
        """Should return root node from XML string."""
        root = _try_parse(self.xml)
        self.assertEqual("a", root.tag)

    def test_try_parse_with_path(self):
        """Should return root node from XML file path."""
        xml_path = os.path.join(self.cwd, "test_try_parse.xml")
        self.tidy_file = xml_path
        with open(xml_path, "w") as xml_file:
            xml_file.write(self.xml)
        root = _try_parse(xml_path)
        self.assertEqual("a", root.tag)

    def test_try_parse_with_bad_path(self):
        """Should raise IOError: file doesn't exist."""
        xml_path = os.path.join(self.cwd, "not_a_real_file.xyz")
        with self.assertRaises(IOError):
            _try_parse(xml_path)

    def test_try_parse_with_bad_string(self):
        """Should raise IOError: string parse failed and its not a path."""
        with self.assertRaises(IOError):
            _try_parse("not valid xml :(")

    def test_try_parse_with_bad_file(self):
        """Should raise XMLSyntaxError: file exists but content is not valid."""
        xml_path = os.path.join(self.cwd, "test_try_parse.xml")
        self.tidy_file = xml_path
        with open(xml_path, "w") as xml_file:
            xml_file.write("not valid xml :(")
        with self.assertRaises(ParseError):
            _try_parse(xml_path)



# vim: ai ts=4 sts=4 et sw=4 encoding=utf-8

from collections import defaultdict


def infinite_dict():
    return defaultdict(infinite_dict)


# The big idea with this class structure is I want to do the
# following:
# translator = Translator()
# translator.add_translation(...)
# translator.translate('How are you?').from('English').to('French')

class _StringWithLanguageTranslator(object):
    def __init__(self, dictionary):
        self._dict = dictionary

    def to_language(self, language):
        if language in self._dict:
            return self._dict[language]
        return None


class _StringTranslator(object):
    def __init__(self, dictionary):
        self._dict = dictionary

    def from_language(self, language):
        dictionary = self._dict[language]
        return _StringWithLanguageTranslator(dictionary)


class Translator(object):
    def __init__(self):
        """
        I'm being super lazy dictionary has to have the form:
        {'yes' : {'English' : {'French' : 'oui'}}}
        """
        self._dict = infinite_dict()
        self._languages = []

    def add_translation(self, string, source_language,
                        destination_language, translated_string):
        for lang in [source_language, destination_language]:
            if lang not in self._languages:
                self._languages.append(lang)
        self._dict[string][source_language][
            destination_language] = translated_string

    def translate(self, string):
        dictionary = self._dict[string]
        return _StringTranslator(dictionary)

    def to_json_dict(self):
        return self._dict

# code used to construct a translator from the excel files from phase II.
# import glob, os
# from xls2json import ExcelReader, print_pyobj_to_json
# from translator import Translator

# translator = Translator()

# def add_dict(d):
#     keys = d.keys()
#     keys.remove(u"English")
#     for key in keys:
#         yield {u"string" : d[u"English"],
#                u"source_language" : u"English",
#                u"destination_language" : key,
#                u"translated_string" : d[key]}

# def add_row(d):
#     assert type(d)==dict, str(d)
#     for k, v in d.items(): 
#         if type(v)==dict and u"English" in v.keys():
#             for result in add_dict(v): yield result

# xls_files = glob.glob( os.path.join("translators", "*", "*.xls") )
# all_translations = []
# for xls_file in xls_files:
#     excel_reader = ExcelReader(xls_file)
#     for sheet_name, list_of_dicts in excel_reader.to_json_dict().items():
#         for d in list_of_dicts:
#             for result in add_row(d):
#                 translator.add_translation(**result)
# print_pyobj_to_json(translator.to_json_dict(), "nigeria.json")



# These tests were being run twice by nose.
# commenting out the imports prevents this.
# from j2x_question_tests import *
# from j2x_test_creation import *
# #from j2x_test_xform_build_preparation import *
# from js2x_test_import_from_json import *
# from j2x_test_instantiation import *
# from json2xform_test import *
# from group_test import *
# from builder_tests import *
# from xls2json_tests import *
# from dump_and_load_tests import *



#!/usr/bin/python
# -*- coding: utf-8 -*-
from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class DoubleColonTranslations(PyxformTestCase):
    def test_langs(self):
        model_contains = """<bind nodeset="/translations/n1""" + \
                         """" readonly="true()" type="string"/>"""
        self.assertPyxformXform(
            name='translations',
            id_string='transl',
            md="""
            | survey |      |      |                |               |
            |        | type | name | label::english | label::french |
            |        | note | n1   | hello          | bonjour       |
            """,
            errored=False,
            itext__contains=[
                '<translation lang="french">',
                '<text id="/translations/n1:label">',
                '<value>bonjour</value>',
                '</text>',
                '</translation>',
                '<translation lang="english">',
                '<text id="/translations/n1:label">',
                '<value>hello</value>',
                '</text>',
                '</translation>',
            ],
            xml__contains=[
                """<label ref="jr:itext('/translations/n1:label')"/>""",
            ],
            model__contains=[model_contains],
        )



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class InvalidSurveyColumnsTests(PyxformTestCase):
    def test_missing_name(self):
        """
        every question needs a name (or alias of name)
        """
        self.assertPyxformXform(
            name='invalidcols',
            ss_structure={'survey': [{'type': 'text',
                                      'label': 'label'}]},
            errored=True,
            error__contains=['no name'],
        )

    def test_missing_name_but_has_alias_of_name(self):
        self.assertPyxformXform(
            name='invalidcols',
            ss_structure={'survey': [{'value': 'q1',
                                      'type': 'text',
                                      'label': 'label'}]},
            errored=False,
        )

    def test_missing_label(self):
        self.assertPyxformXform(
            name="invalidcols",
            ss_structure={'survey': [{'type': 'text',
                                      'name': 'q1'}]},
            errored=True,
            error__contains=['no label or hint'],
        )

    def test_column_case(self):
        """
        Ensure that column name is case insensitive
        """
        self.assertPyxformXform(
            name="mixedcasecolumns",
            md="""
            | Survey |         |         |               |
            |        | Type    | name    | Label         |
            |        | text    | Name    | the name      |
            |        | integer | age     | the age       |
            |        | text    | gender  | the gender    |
            """,
            errored=False,
            debug=True
        )


class InvalidChoiceSheetColumnsTests(PyxformTestCase):
    def _simple_choice_ss(self, choice_sheet=None):
        if choice_sheet is None:
            choice_sheet = []
        return {'survey': [{'type': 'select_one l1',
                            'name': 'l1choice',
                            'label': 'select one from list l1'}],
                'choices': choice_sheet}

    def test_valid_choices_sheet_passes(self):
        self.assertPyxformXform(
            name='valid_choices',
            ss_structure=self._simple_choice_ss([
                {'list_name': 'l1',
                    'name': 'c1',
                    'label': 'choice 1'},
                {'list_name': 'l1',
                    'name': 'c2',
                    'label': 'choice 2'}]),
            errored=False,
            )

    def test_invalid_choices_sheet_fails(self):
        self.assertPyxformXform(
            name='missing_name',
            ss_structure=self._simple_choice_ss([
                {'list_name': 'l1',
                    'label': 'choice 1'},
                {'list_name': 'l1',
                    'label': 'choice 2'},
                ]),
            errored=True,
            error__contains=['option with no name'],
            )

    def test_missing_list_name(self):
        self.assertPyxformXform(
            name='missing_list_name',
            ss_structure=self._simple_choice_ss([
                {'bad_column': 'l1',
                    'name': 'l1c1',
                    'label': 'choice 1'},
                {'bad_column': 'l1',
                    'name': 'l1c1',
                    'label': 'choice 2'},
                ]),
            debug=True,
            errored=True,
            # some basic keywords that should be in the error:
            error__contains=[
                'choices',
                'name',
                'list name',
            ])


class AliasesTests(PyxformTestCase):
    def test_value_and_name(self):
        '''
        confirm that both 'name' and 'value' columns of choice list work
        '''
        for name_alias in ['name', 'value']:
            self.assertPyxformXform(
                name="aliases",
                md="""
                | survey  |               |                |            |
                |         | type          | name           | label      |
                |         | select_one yn | q1             | Question 1 |
                | choices |               |                |            |
                |         | list name     | %(name_alias)s | label      |
                |         | yn            | yes            | Yes        |
                |         | yn            | no             | No         |
                """ % ({
                        u'name_alias': name_alias
                    }),
                instance__contains=[
                    '<q1/>',
                    ],
                model__contains=[
                    '<bind nodeset="/aliases/q1" type="select1"/>',
                    ],
                xml__contains=[
                    '<select1 ref="/aliases/q1">',
                    '<value>yes</value>',
                    '<value>no</value>',
                    '</select1>',
                    ])
''' # uncomment when re-implemented
    # TODO: test that this fails for the correct reason
    def test_conflicting_aliased_values_raises_error(self):
        # example:
        # an xlsform has {"name": "q_name", "value": "q_value"}
        # should not compile because "name" and "value" columns are aliases

        self.assertPyxformXform(
            # debug=True,
            name="aliases",
            md="""
            | survey |      |        |         |            |
            |        | type | name   | value   | label      |
            |        | text | q_name | q_value | Question 1 |
            """,
            errored=True,
        )
'''



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class TestLoop(PyxformTestCase):
    def test_loop(self):
        self.assertPyxformXform(
            name="test_loop",
            md="""
                | survey |              |           |                |        |
                |        |     type     |    name   | bind:relevant  | label  |
                |        | begin repeat | for-block |                | Oh HAI |
                |        | string       | input     | (${done}='no') | HI HI  |
                |        | string       | done      |                | DONE?  |
                |        | end repeat   |           |                |        |
                """,
            instance__contains=[
                '<for-block jr:template="">',
                '</for-block>',
                ],
            model__contains=[
                """<bind nodeset="/test_loop/for-block/input" """
                """relevant="( /test_loop/for-block/done ='no')" """
                """type="string"/>"""
                ],
            xml__contains=[
                '<group ref="/test_loop/for-block">',
                '<label>Oh HAI</label>',
                '</group>',
            ],
        )



"""
Testing preparation of values for XForm exporting
"""
from unittest import TestCase

from pyxform import *


class Json2XformExportingPrepTests(TestCase):
    
    def test_dictionary_consolidates_duplicate_entries(self):
        
        yes_or_no_dict_array = [
            {"label": {"French": "Oui", "English": "Yes"}, "name": "yes"},
            {"label": {"French": "Non", "English": "No"}, "name": "no"}
            ]

        first_yesno_question = MultipleChoiceQuestion(
            name="yn_q1", options=yes_or_no_dict_array, type="select one")
        second_yesno_question = MultipleChoiceQuestion(
            name="yn_q2", options=yes_or_no_dict_array, type="select one")
        
        s = Survey(name="yes_or_no_tests")
        s.add_child(first_yesno_question)
        s.add_child(second_yesno_question)
        
        # begin the processes in survey.to_xml()
        # 1. validate()
        s.validate()
        
        # 2. survey._build_options_list_from_descendants()
        # options_list = s._build_options_list_from_descendants()
        # Is this method called somewhere else now?
        
        # desired_options_list = [first_yesno_question.children]

        # todo: we need to think about whether we care about
        # consolidating these options lists.
        # self.assertEqual(options_list, desired_options_list)
        # self.assertEqual(first_yesno_question._option_list_index_number, 0)
        # self.assertEqual(second_yesno_question._option_list_index_number, 0)



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class MissingHeaders(PyxformTestCase):
    """
    when survey and choices columns are missing headers, it is helpful to see
    an error message that prompts the user to include necessary headers.
    """
    def test_missing_survey_headers(self):
        self.assertPyxformXform(
            md="""
            | survey |                 |    |
            |        | select_one list | S1 |
            """,
            errored=True,
            error__contains=[
                'missing important column headers',
            ])

    def test_missing_choice_headers(self):
        self.assertPyxformXform(
            md="""
            | survey  |                 |          |      |
            |         | type            | label    | name |
            |         | select_one list | S1       | s1   |
            | choices |                 |          |      |
            |         | list            | option a | a    |
            |         | list            | option b | b    |
            """,
            errored=True,
            error__contains=[
                "has columns 'list name', 'name', and 'label'",
            ])



"""
This file contains constants that correspond with the property names in the
json survey format. (@see json_form_schema.json) These names are to be shared
between X2json and json2Y programs. By putting them in a shared file,
the literal names can be easily changed, typos can be avoided, and references
are easier to find.
"""
# TODO: Replace matching strings in the json2xforms code (builder.py,
# survey.py, survey_element.py, question.py) with these constants

TYPE = u"type"
TITLE = u"title"
NAME = u"name"
ID_STRING = u"id_string"
SMS_KEYWORD = u"sms_keyword"
SMS_FIELD = u"sms_field"
SMS_OPTION = u"sms_option"
SMS_SEPARATOR = u"sms_separator"
SMS_ALLOW_MEDIA = u"sms_allow_media"
SMS_DATE_FORMAT = u"sms_date_format"
SMS_DATETIME_FORMAT = u"sms_datetime_format"
SMS_RESPONSE = u"sms_response"
VERSION = u"version"
PUBLIC_KEY = u"public_key"
SUBMISSION_URL = u"submission_url"
DEFAULT_LANGUAGE = u"default_language"
LABEL = u"label"
HINT = u"hint"
STYLE = u"style"
ATTRIBUTE = u"attribute"

BIND = u"bind"  # TODO: What should I do with the nested types? (readonly and relevant) # noqa
MEDIA = u"media"
CONTROL = u"control"
APPEARANCE = u"appearance"

LOOP = u"loop"
COLUMNS = u"columns"

REPEAT = u"repeat"
GROUP = u"group"
CHILDREN = u"children"

SELECT_ONE = u"select one"
SELECT_ALL_THAT_APPLY = u"select all that apply"
CHOICES = u"choices"

# XLS Specific constants
LIST_NAME = u"list name"
CASCADING_SELECT = u"cascading_select"
TABLE_LIST = u"table-list"  # hyphenated because it goes in appearance, and convention for appearance column is dashes # noqa

# The following are the possible sheet names:
SURVEY = u"survey"
SETTINGS = u"settings"
# These sheet names are for list sheets
CHOICES_AND_COLUMNS = u"choices and columns"
CASCADING_CHOICES = u"cascades"

OSM = u"osm"
OSM_TYPE = u"binary"

NAMESPACES = u'namespaces'



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class ExternalCSVInstancesTest(PyxformTestCase):
    def test_external_csv_instances(self):
        # re: https://github.com/XLSForm/pyxform/issues/30
        self.assertPyxformXform(
            name="ecsv",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          |
            |        | select_one_from_file cities.csv              | city           | City           |
            |        | select_multiple_from_file neighbourhoods.csv | neighbourhoods | Neighbourhoods |
            """,  # noqa
            xml__contains=[
                """<instance id="cities" src="jr://file-csv/cities.csv">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select1 ref="/ecsv/city">',
                '<itemset nodeset="instance(\'cities\')/root/item">',
                """<instance id="neighbourhoods" src="jr://file-csv/neighbourhoods.csv">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select ref="/ecsv/neighbourhoods">',
                '<itemset nodeset="instance(\'neighbourhoods\')/root/item">',
            ],
            run_odk_validate=True,
        )

    def test_external_csv_instances_w_choice_filter(self):
        # re: https://github.com/XLSForm/pyxform/issues/30
        self.assertPyxformXform(
            name="ecsv",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          | choice_filter |
            |        | select_one_from_file cities.csv              | city           | City           |               |
            |        | select_multiple_from_file neighbourhoods.csv | neighbourhoods | Neighbourhoods | city=${city}  |
            """,  # noqa
            xml__contains=[
                """<instance id="cities" src="jr://file-csv/cities.csv">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select1 ref="/ecsv/city">',
                """<instance id="neighbourhoods" src="jr://file-csv/neighbourhoods.csv">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select ref="/ecsv/neighbourhoods">',
                '<itemset nodeset="instance(\'neighbourhoods\')/root/item[city= /ecsv/city ]">',  # noqa
            ],
            run_odk_validate=True,
        )


class ExternalXMLInstancesTest(PyxformTestCase):
    def test_external_xml_instances(self):
        # re: https://github.com/XLSForm/pyxform/issues/30
        self.assertPyxformXform(
            name="exml",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          |
            |        | select_one_from_file cities.xml              | city           | City           |
            |        | select_multiple_from_file neighbourhoods.xml | neighbourhoods | Neighbourhoods |
            """,  # noqa
            xml__contains=[
                """<instance id="cities" src="jr://file/cities.xml">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select1 ref="/exml/city">',
                '<itemset nodeset="instance(\'cities\')/root/item">',
                """<instance id="neighbourhoods" src="jr://file/neighbourhoods.xml">
        <root>
          <item>
            <name>_</name>
            <label>_</label>
          </item>
        </root>
      </instance>""",  # noqa
                '<select ref="/exml/neighbourhoods">',
                '<itemset nodeset="instance(\'neighbourhoods\')/root/item">',
            ],
            run_odk_validate=True,
        )


class InvalidExternalFileInstancesTest(PyxformTestCase):
    def test_external_other_extension_instances(self):
        # re: https://github.com/XLSForm/pyxform/issues/30
        self.assertPyxformXform(
            name="epdf",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          |
            |        | select_one_from_file cities.pdf              | city           | City           |
            |        | select_multiple_from_file neighbourhoods.pdf | neighbourhoods | Neighbourhoods |
            """,  # noqa
            errored=True,
            error_contains=["should be a choices sheet in this xlsform"],
        )

    def test_external_choices_sheet_included_instances(self):
        # re: https://github.com/XLSForm/pyxform/issues/30
        self.assertPyxformXform(
            name="epdf",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          |
            |        | select_one_from_file cities.pdf              | city           | City           |
            |        | select_multiple_from_file neighbourhoods.pdf | neighbourhoods | Neighbourhoods |

            | choices |
            |         | list name | name  | label |
            |         | fruits    | apple | Apple |
            """,  # noqa
            errored=True,
            error__contains=["List name not in choices sheet: cities.pdf"],
        )


class ExternalCSVInstancesBugsTest(PyxformTestCase):
    def test_non_existent_itext_reference(self):
        # re: https://github.com/XLSForm/pyxform/issues/80
        self.assertPyxformXform(
            name="ecsv",
            md="""
            | survey |                                              |                |                |
            |        | type                                         | name           | label          |
            |        | select_one_from_file cities.csv              | city           | City           |
            |        | select_multiple_from_file neighbourhoods.csv | neighbourhoods | Neighbourhoods |
            """,  # noqa
            xml__contains=[
                """<itemset nodeset="instance('cities')/root/item">
        <value ref="name"/>
        <label ref="label"/>
      </itemset>"""
            ],
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import codecs
import os
import re
import tempfile
import xml.etree.ElementTree as ETree
from unittest import TestCase

from pyxform.builder import create_survey_element_from_dict
from pyxform.errors import PyXFormError
from pyxform.odk_validate import ODKValidateError, check_xform
from pyxform.survey import nsmap
from pyxform.tests_v1.test_utils.md_table import md_table_to_ss_structure
from pyxform.xls2json import workbook_to_json


class PyxformTestError(Exception):
    pass


class PyxformMarkdown(object):
    """Transform markdown formatted xlsform to a pyxform survey object"""

    def md_to_pyxform_survey(self, md_raw, kwargs=None, autoname=True):
        if kwargs is None:
            kwargs = {}
        if autoname:
            kwargs = self._autonameInputs(kwargs)
        _md = []
        for line in md_raw.split('\n'):
            if re.match(r'^\s+\#', line):
                # ignore lines which start with pound sign
                continue
            elif re.match(r'^(.*)(\#[^\|]+)$', line):
                # keep everything before the # outside of the last occurrence
                # of |
                _md.append(
                    re.match(r'^(.*)(\#[^\|]+)$', line).groups()[0].strip())
            else:
                _md.append(line.strip())
        md = '\n'.join(_md)

        if kwargs.get('debug'):
            print(md)

        def list_to_dicts(arr):
            headers = arr[0]

            def _row_to_dict(row):
                out_dict = {}
                for i in range(0, len(row)):
                    col = row[i]
                    if col not in [None, '']:
                        out_dict[headers[i]] = col
                return out_dict

            return [_row_to_dict(r) for r in arr[1:]]

        sheets = {}
        for sheet, contents in md_table_to_ss_structure(md):
            sheets[sheet] = list_to_dicts(contents)

        return self._ss_structure_to_pyxform_survey(sheets, kwargs)

    def _ss_structure_to_pyxform_survey(self, ss_structure, kwargs):
        # using existing methods from the builder
        imported_survey_json = workbook_to_json(ss_structure)
        # ideally, when all these tests are working, this would be
        # refactored as well
        survey = create_survey_element_from_dict(imported_survey_json)
        survey.name = kwargs.get('name')
        survey.title = kwargs.get('title')
        survey.id_string = kwargs.get('id_string')

        return survey

    def _autonameInputs(self, kwargs):
        """
        title and name are necessary for surveys, but not always convenient to
        include in test cases, so this will pull a default value
        from the stack trace.
        """
        test_name_root = 'pyxform'
        if 'name' not in kwargs.keys():
            kwargs['name'] = test_name_root + '_autotestname'
        if 'title' not in kwargs.keys():
            kwargs['title'] = test_name_root + "_autotesttitle"
        if 'id_string' not in kwargs.keys():
            kwargs['id_string'] = test_name_root + "_autotest_id_string"

        return kwargs


class PyxformTestCase(PyxformMarkdown, TestCase):
    def assertPyxformXform(self, **kwargs):
        """
        PyxformTestCase.assertPyxformXform() named arguments:
        -----------------------------------------------------

        one of these possible survey input types
          * md: (str) a markdown formatted xlsform (easy to read in code)
                [consider a plugin to help with formatting md tables,
                 e.g. https://github.com/vkocubinsky/SublimeTableEditor]
          * ss_structure: (dict) a python dictionary with sheets and their
                contents. best used in cases where testing whitespace and
                cells' type is important
          * survey: (pyxform.survey.Survey) easy for reuse within a test
          # Note: XLS is not implemented at this time. You can use builder to
          create a pyxform Survey object

        one or many of these string "matchers":
          * xml__contains: an array of strings which exist in the
                resulting xml
          * error__contains: a list of strings which should exist in
                the error
          * odk_validate_error__contains: list of strings; run_odk_validate
                must be set

        optional other parameters passed to pyxform:
          * errored: (bool) if the xlsform is not supposed to compile,
                this must be True
          * name: (str) a valid xml tag to be used as the form name
          * id_string: (str)
          * title: (str)
          * run_odk_validate: (bool) when True, runs ODK Validate process
                Default value = False because it slows down tests
        """
        debug = kwargs.get('debug', False)
        expecting_invalid_survey = kwargs.get('errored', False)
        errors = []
        xml_nodes = {}

        run_odk_validate = kwargs.get('run_odk_validate', False)
        odk_validate_error__contains = \
            kwargs.get('odk_validate_error__contains', [])

        try:
            if 'md' in kwargs.keys():
                kwargs = self._autonameInputs(kwargs)
                survey = self.md_to_pyxform_survey(kwargs.get('md'), kwargs)
            elif 'ss_structure' in kwargs.keys():
                kwargs = self._autonameInputs(kwargs)
                survey = self._ss_structure_to_pyxform_survey(
                    kwargs.get('ss_structure'), kwargs)
            else:
                survey = kwargs.get("survey")

            xml = survey._to_pretty_xml()
            root = ETree.fromstring(xml.encode('utf-8'))

            # Ensure all namespaces are present, even if unused
            root.attrib.update(**nsmap)

            xml_nodes['xml'] = root

            def _pull_xml_node_from_root(element_selector):
                ns = 'http://www.w3.org/2002/xforms'
                _r = root.findall('.//n:%s' % element_selector,
                                  namespaces={'n': ns})
                if len(_r) == 0:
                    return False
                else:
                    return _r[0]

            for _n in ['model', 'instance', 'itext']:
                xml_nodes[_n] = _pull_xml_node_from_root(_n)
            if debug:
                print(xml)
            if run_odk_validate:
                # On Windows, NamedTemporaryFile must be opened exclusively.
                # So it must be explicitly created, opened, closed, and removed
                tmp = tempfile.NamedTemporaryFile(suffix='.xml', delete=False)
                tmp.close()
                try:
                    with codecs.open(
                            tmp.name, mode="w", encoding="utf-8") as fp:
                        if '_xml_append' in kwargs:
                            xml += kwargs['_xml_append']
                        fp.write(xml)
                        fp.close()
                    check_xform(tmp.name)
                finally:
                    # Clean up the temporary file
                    os.remove(tmp.name)
                    assert not os.path.isfile(tmp.name)

                if len(odk_validate_error__contains) > 0:
                    raise PyxformTestError(
                        "ODKValidateError was not raised"
                        )
        except PyXFormError as e:
            survey = False
            errors = [str(e)]
            if debug:
                print("<xml unavailable />")
                print("ERROR: '%s'" % errors[0])
        except ODKValidateError as e:
            if len(odk_validate_error__contains) is 0:
                raise PyxformTestError("ODK Validate error was thrown but " +
                                       "'odk_validate_error__contains'" +
                                       " was empty:" + e.message)
            for v_err in odk_validate_error__contains:
                self.assertContains(e.message, v_err,
                                    msg_prefix='odk_validate_error__contains')
        else:
            survey = True

        if survey:
            def _check_contains(keyword):
                contains_str = '%s__contains' % keyword

                def check_content(content):
                    text_arr = kwargs[contains_str]
                    for text in text_arr:
                        self.assertContains(content, text, msg_prefix=keyword)

                return contains_str, check_content

            if 'body_contains' in kwargs or 'body__contains' in kwargs:
                raise SyntaxError("Invalid parameter: 'body__contains'."
                                  "Use 'xml__contains' instead")

            for code in ['xml', 'instance', 'model', 'itext']:
                (code__str, checker) = _check_contains(code)
                if kwargs.get(code__str):
                    checker(ETree.tostring(
                        xml_nodes[code], encoding='utf-8').decode('utf-8'))
                bad_kwarg = '%s_contains' % code
                if bad_kwarg in kwargs:
                    good_kwarg = '%s__contains' % code
                    raise SyntaxError(("'%s' is not a valid parameter. "
                                      "Use double underscores: '%s'") %
                                      (bad_kwarg, good_kwarg))

        if survey is False and expecting_invalid_survey is False:
            raise PyxformTestError(
                "Expected valid survey but compilation failed. "
                "Try correcting the error with 'debug=True', "
                "setting 'errored=True', "
                "and or optionally 'error__contains=[...]'")
        elif survey and expecting_invalid_survey:
            raise PyxformTestError("Expected survey to be invalid.")

        if 'error__contains' in kwargs:
            joined_error = '\n'.join(errors)
            for text in kwargs['error__contains']:
                self.assertContains(joined_error, text,
                                    msg_prefix="error__contains")

    def _assert_contains(self, content, text, msg_prefix):
        if msg_prefix:
            msg_prefix += ": "

        # Account for space in self-closing tags
        text_repr = repr(text)
        content = content.replace(" />", "/>")
        real_count = content.count(text)

        return text_repr, real_count, msg_prefix

    def assertContains(self, content, text, count=None, msg_prefix=''):
        """
        FROM: django source- testcases.py

        Asserts that ``text`` occurs ``count`` times in the content string.
        If ``count`` is None, the count doesn't matter - the assertion is
        true if the text occurs at least once in the content.
        """
        text_repr, real_count, msg_prefix = self._assert_contains(
            content, text, msg_prefix)

        if count is not None:
            self.assertEqual(
                real_count, count,
                msg_prefix + "Found %d instances of %s in content"
                " (expected %d)" % (real_count, text_repr, count))
        else:
            self.assertTrue(
                real_count != 0,
                msg_prefix + "Couldn't find %s in content" % text_repr)

    def assertNotContains(self, content, text, msg_prefix=''):
        """
        Asserts that a content indicates that some content was retrieved
        successfully, (i.e., the HTTP status code was as expected), and that
        ``text`` doesn't occurs in the content of the content.
        """
        text_repr, real_count, msg_prefix = self._assert_contains(
            content, text, msg_prefix)

        self.assertEqual(
            real_count, 0,
            msg_prefix + "Response should not contain %s" % text_repr)



from pyxform import constants

# Aliases:
# Ideally aliases should resolve to elements in the json form schema

# select, control and settings alias keys used for parsing,
# which is why self mapped keys are necessary.

control = {
    u"group": constants.GROUP,
    u"lgroup": constants.REPEAT,
    u"repeat": constants.REPEAT,
    u"loop": constants.LOOP,
    u"looped group": constants.REPEAT
}
select = {
    u"add select one prompt using": constants.SELECT_ONE,
    u"add select multiple prompt using": constants.SELECT_ALL_THAT_APPLY,
    u"select all that apply from": constants.SELECT_ALL_THAT_APPLY,
    u"select one from": constants.SELECT_ONE,
    u"select1": constants.SELECT_ONE,
    u"select_one": constants.SELECT_ONE,
    u"select one": constants.SELECT_ONE,
    u"select_multiple": constants.SELECT_ALL_THAT_APPLY,
    u"select all that apply": constants.SELECT_ALL_THAT_APPLY,
    u"select_one_external": u"select one external",
    u"select_one_from_file": constants.SELECT_ONE,
    u"select_multiple_from_file": constants.SELECT_ALL_THAT_APPLY,
    u"select one from file": constants.SELECT_ONE,
    u"select multiple from file": constants.SELECT_ALL_THAT_APPLY,
}
cascading = {
    u'cascading select': constants.CASCADING_SELECT,
    u'cascading_select': constants.CASCADING_SELECT,
}
settings_header = {
    u"form_title": constants.TITLE,
    u"set form title": constants.TITLE,
    u"form_id": constants.ID_STRING,
    u"sms_keyword": constants.SMS_KEYWORD,
    u"sms_separator": constants.SMS_SEPARATOR,
    u"sms_allow_media": constants.SMS_ALLOW_MEDIA,
    u"sms_date_format": constants.SMS_DATE_FORMAT,
    u"sms_datetime_format": constants.SMS_DATETIME_FORMAT,
    u"set form id": constants.ID_STRING,
    u"public_key": constants.PUBLIC_KEY,
    u"submission_url": constants.SUBMISSION_URL
}
# TODO: Check on bind prefix approach in json.
# Conversion dictionary from user friendly column names to meaningful values
survey_header = {
    u"Label": u"label",
    u"Name": u"name",
    u"SMS Field": constants.SMS_FIELD,
    u"SMS Option": constants.SMS_OPTION,
    u"SMS Sepatator": constants.SMS_SEPARATOR,
    u"SMS Allow Media": constants.SMS_ALLOW_MEDIA,
    u"SMS Date Format": constants.SMS_DATE_FORMAT,
    u"SMS DateTime Format": constants.SMS_DATETIME_FORMAT,
    u"SMS Response": constants.SMS_RESPONSE,
    u"Type": u"type",
    u"List_name": u"list_name",
    # u"repeat_count": u"jr:count",  duplicate key
    u"read_only": u"bind::readonly",
    u"readonly": u"bind::readonly",
    u"relevant": u"bind::relevant",
    u"caption": constants.LABEL,
    u"appearance": u"control::appearance",  # TODO: this is also an issue
    u"relevance": u"bind::relevant",
    u"required": u"bind::required",
    u"constraint": u"bind::constraint",
    u"constraining message": u"bind::jr:constraintMsg",
    u"constraint message": u"bind::jr:constraintMsg",
    u"constraint_message": u"bind::jr:constraintMsg",
    u"calculation": u"bind::calculate",
    u"calculate": u"bind::calculate",
    u"command": constants.TYPE,
    u"tag": constants.NAME,
    u"value": constants.NAME,
    u"image": u"media::image",
    u"audio": u"media::audio",
    u"video": u"media::video",
    u"count": u"control::jr:count",
    u"repeat_count": u"control::jr:count",
    u"jr:count": u"control::jr:count",
    u"autoplay": u"control::autoplay",
    u"rows": u"control::rows",
    # New elements that have to go into itext elements:
    u"noAppErrorString": u"bind::jr:noAppErrorString",
    u"no_app_error_string": u"bind::jr:noAppErrorString",
    u"requiredMsg": u"bind::jr:requiredMsg",
    u"required_message": u"bind::jr:requiredMsg",
    u"required message": u"bind::jr:requiredMsg",
    u"body": u"control",
    u"parameters": u"parameters",
}
list_header = {
    u"caption": constants.LABEL,
    u"list_name": constants.LIST_NAME,
    u"value": constants.NAME,
    u"image": u"media::image",
    u"audio": u"media::audio",
    u"video": u"media::video"
}
# Note that most of the type aliasing happens in all.xls
_type = {
    u"imei": u"deviceid",
    u"image": u"photo",
    u"add image prompt": u"photo",
    u"add photo prompt": u"photo",
    u"add audio prompt": u"audio",
    u"add video prompt": u"video",
    u"add file prompt": u"file"
}
yes_no = {
    "yes": True,
    "Yes": True,
    "YES": True,
    "true": True,
    "True": True,
    "TRUE": True,
    "true()": True,
    "no": False,
    "No": False,
    "NO": False,
    "false": False,
    "False": False,
    "FALSE": False,
    "false()": False,
}
label_optional_types = [
    u"deviceid",
    u"phonenumber",
    u"simserial",
    u"calculate",
    u"start",
    u"end",
    u"today"
]
osm = {
    u"osm": constants.OSM_TYPE
}



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class RangeWidgetTest(PyxformTestCase):
    def test_range_type(self):
        # parameters column
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | start=1 end=10 step=1 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

        # mixed case parameters
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | Start=3 End=14 STEP=2 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="14" ref="/data/level" start="3" step="2">'],
        )

    def test_range_type_defaults(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale |                     |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | end=20              |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="20" ref="/data/level" start="1" step="1">'],
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |
            |        | type   |   name   | label |
            |        | range  |   level  | Scale |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

    def test_range_type_float(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | start=0.5 end=5.0 step=0.5 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="decimal"/>',
                '<range end="5.0" ref="/data/level" start="0.5" step="0.5">'],
        )

    def test_range_type_invvalid_parameters(self):
        # 'increment' is an invalid property
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                        |
            |        | type   |   name   | label | parameters             |
            |        | range  |   level  | Scale | increment=0.5 end=21.5 |
            """,
            errored=True,
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | start=0.5 end=X step=0.5 |
            """,
            errored=True,
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                     |
            |        | type   |   name   | label | parameters          |
            |        | range  |   level  | Scale | start               |
            """,
            errored=True,
        )

    def test_range_semicolon_separator(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                       |
            |        | type   |   name   | label | parameters            |
            |        | range  |   level  | Scale | start=1;end=10;step=1 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

    def test_range_comma_separator(self):
        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                       |
            |        | type   |   name   | label | parameters            |
            |        | range  |   level  | Scale | start=1,end=10,step=1 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                           |
            |        | type   |   name   | label | parameters                |
            |        | range  |   level  | Scale | start=1 , end=10 , step=1 |
            """,
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="1">'],
        )

        self.assertPyxformXform(
            name="data",
            md="""
            | survey |        |          |       |                                 |
            |        | type   |   name   | label | parameters                      |
            |        | range  |   level  | Scale | start = 1 , end = 10 , step = 2 |
            """,  # noqa
            xml__contains=[
                '<bind nodeset="/data/level" type="int"/>',
                '<range end="10" ref="/data/level" start="1" step="2">'],
        )



from pyxform.tests_v1.pyxform_test_case import PyxformTestCase


class AuditTest(PyxformTestCase):
    def test_audit(self):
        self.assertPyxformXform(
            name="meta_audit",
            md="""
            | survey |        |          |       |
            |        | type   |   name   | label |
            |        | audit  |   audit  |       |
            """,
            xml__contains=[
                '<meta>',
                '<audit/>',
                '</meta>',
                '<bind nodeset="/meta_audit/meta/audit" type="binary"/>'],
        )

    def test_audit_random_name(self):
        self.assertPyxformXform(
            name="meta_audit",
            md="""
            | survey |        |          |       |
            |        | type   |   name   | label |
            |        | audit  |   bobby  |       |
            """,
            errored=True,
            error__contains=['Audits must always be named \'audit.\''],
        )

    def test_audit_blank_name(self):
        self.assertPyxformXform(
            name="meta_audit",
            md="""
            | survey |        |          |       |
            |        | type   |   name   | label |
            |        | audit  |          |       |
            """,
            xml__contains=[
                '<meta>',
                '<audit/>',
                '</meta>',
                '<bind nodeset="/meta_audit/meta/audit" type="binary"/>'],
        )


import re


def _strp_cell(cell):
    val = cell.strip()
    if val == '':
        return None

    return val


def _extract_array(mdtablerow):
    match = re.match('\s*\|(.*)\|\s*', mdtablerow)
    if match:
        mtchstr = match.groups()[0]
        if re.match('^[\|-]+$', mtchstr):
            return False
        else:
            return [_strp_cell(c) for c in mtchstr.split('|')]

    return False


def _is_null_row(r_arr):
    for cell in r_arr:
        if cell is not None:
            return False

    return True


def md_table_to_ss_structure(mdstr):
    ss_arr = []
    for item in mdstr.split('\n'):
        arr = _extract_array(item)
        if arr:
            ss_arr.append(arr)
    sheet_name = False
    sheet_arr = False
    sheets = []
    for row in ss_arr:
        if row[0] is not None:
            if sheet_arr:
                sheets.append((sheet_name, sheet_arr,))
            sheet_arr = []
            sheet_name = row[0]
        excluding_first_col = row[1:]
        if sheet_name and not _is_null_row(excluding_first_col):
            sheet_arr.append(excluding_first_col)
    sheets.append((sheet_name, sheet_arr,))

    return sheets



from __future__ import print_function
from unittest import TestCase
from pyxform.xls2json_backends import convert_file_to_csv_string
from pyxform.tests import utils


class BackendUtilsTests(TestCase):
    def test_xls_to_csv(self):
        specify_other_xls = utils.path_to_text_fixture("specify_other.xls")
        converted_xls = convert_file_to_csv_string(specify_other_xls)
        specify_other_csv = utils.path_to_text_fixture("specify_other.csv")
        converted_csv = convert_file_to_csv_string(specify_other_csv)
        print("csv:")
        print(converted_csv)
        print("xls:")
        print(converted_xls)
        self.assertEqual(converted_csv, converted_xls)




import codecs
import os
import re
import tempfile
import xml.etree.ElementTree as ETree
from collections import defaultdict
from datetime import datetime

from pyxform import constants
from pyxform.errors import PyXFormError
from pyxform.instance import SurveyInstance
from pyxform.odk_validate import check_xform
from pyxform.question import Question
from pyxform.section import Section
from pyxform.survey_element import SurveyElement
from pyxform.utils import PatchedText, basestring, node, unicode

nsmap = {
    u"xmlns": u"http://www.w3.org/2002/xforms",
    u"xmlns:h": u"http://www.w3.org/1999/xhtml",
    u"xmlns:ev": u"http://www.w3.org/2001/xml-events",
    u"xmlns:xsd": u"http://www.w3.org/2001/XMLSchema",
    u"xmlns:jr": u"http://openrosa.org/javarosa",
    u"xmlns:orx": u"http://openrosa.org/xforms"
    }

for prefix, uri in nsmap.items():
    prefix = prefix.replace("xmlns", "").replace(":", "")
    ETree.register_namespace(prefix, uri)


class Survey(Section):

    FIELDS = Section.FIELDS.copy()
    FIELDS.update(
        {
            u"_xpath": dict,
            u"_created": datetime.now,  # This can't be dumped to json
            u"title": unicode,
            u"id_string": unicode,
            u"sms_keyword": unicode,
            u"sms_separator": unicode,
            u"sms_allow_media": bool,
            u"sms_date_format": unicode,
            u"sms_datetime_format": unicode,
            u"sms_response": unicode,
            u"file_name": unicode,
            u"default_language": unicode,
            u"_translations": dict,
            u"submission_url": unicode,
            u"public_key": unicode,
            u"instance_xmlns": unicode,
            u"version": unicode,
            u"choices": dict,
            u"style": unicode,
            u"attribute": dict,
            u"namespaces": unicode,
        }
    )

    def validate(self):
        if self.id_string in [None, 'None']:
            raise PyXFormError('Survey cannot have an empty id_string')
        super(Survey, self).validate()
        self._validate_uniqueness_of_section_names()

    def _validate_uniqueness_of_section_names(self):
        section_names = []
        for e in self.iter_descendants():
            if isinstance(e, Section):
                if e.name in section_names:
                    raise PyXFormError(
                        "There are two sections with the name %s." % e.name)
                section_names.append(e.name)

    def get_nsmap(self):
        """Add additional namespaces"""
        namespaces = getattr(self, constants.NAMESPACES, None)

        if namespaces and isinstance(namespaces, basestring):
            nslist = [
                ns.split('=') for ns in namespaces.split()
                if len(ns.split('=')) == 2 and ns.split('=')[0] != ''
            ]
            xmlns = u'xmlns:'
            nsmap.update(dict([
                (xmlns + k, v.replace('"', '').replace("'", ""))
                for k, v in nslist if xmlns + k not in nsmap
            ]))

        return nsmap

    def xml(self):
        """
        calls necessary preparation methods, then returns the xml.
        """
        self.validate()
        self._setup_xpath_dictionary()
        body_kwargs = {}
        if hasattr(self, constants.STYLE) and getattr(
                self, constants.STYLE):
            body_kwargs['class'] = getattr(
                self, constants.STYLE)
        nsmap = self.get_nsmap()

        return node(u"h:html",
                    node(u"h:head",
                         node(u"h:title", self.title),
                         self.xml_model()
                         ),
                    node(u"h:body", *self.xml_control(), **body_kwargs),
                    **nsmap
                    )

    def _generate_static_instances(self):
        """
        Generates <instance> elements for static data
        (e.g. choices for select type questions)
        """
        for list_name, choice_list in self.choices.items():
            instance_element_list = []
            for idx, choice in zip(range(len(choice_list)), choice_list):
                choice_element_list = []
                # Add a unique id to the choice element incase there is itext
                # it refrences
                itext_id = '-'.join(['static_instance', list_name, str(idx)])
                choice_element_list.append(node("itextId", itext_id))

                for choicePropertyName, choicePropertyValue in choice.items():
                    if isinstance(choicePropertyValue, basestring) \
                            and choicePropertyName != 'label':
                        choice_element_list.append(
                            node(choicePropertyName,
                                 unicode(choicePropertyValue))
                        )
                instance_element_list.append(node("item",
                                                  *choice_element_list))
            yield node("instance", node("root", *instance_element_list),
                       id=list_name)

    def _generate_pulldata_instances(self):
        pulldata = []
        for i in self.iter_descendants():
            if 'calculate' in i['bind']:
                calculate = i['bind']['calculate']
                if calculate.startswith('pulldata('):
                    pieces = calculate.split('"') \
                        if '"' in calculate else calculate.split("'")
                    if len(pieces) > 1 and pieces[1] not in pulldata:
                        csv_id = pieces[1]
                        pulldata.append(csv_id)

                        yield node("instance", id=csv_id,
                                   src="jr://file-csv/{}.csv".format(csv_id)
                                   )

    def _generate_from_file_instances(self):
        for i in self.iter_descendants():
            itemset = i.get('itemset')
            if itemset and \
                    (itemset.endswith('.csv') or itemset.endswith('.xml')):
                file_id, ext = os.path.splitext(itemset)
                uri = 'jr://%s/%s' % (
                    'file' if ext == '.xml' else "file-%s" % ext[1:],
                    itemset)
                yield node(
                    "instance",
                    node("root",
                         node("item", node("name", "_"), node("label", "_"))),
                    id=file_id,
                    src=uri
                )

    def xml_model(self):
        """
        Generate the xform <model> element
        """
        self._setup_translations()
        self._setup_media()
        self._add_empty_translations()

        model_children = []
        if self._translations:
            model_children.append(self.itext())
        model_children += [node("instance", self.xml_instance())]
        model_children += list(self._generate_static_instances())
        model_children += list(self._generate_pulldata_instances())
        model_children += list(self._generate_from_file_instances())
        model_children += self.xml_bindings()

        if self.submission_url or self.public_key:
            submission_attrs = dict()
            if self.submission_url:
                submission_attrs["action"] = self.submission_url
            if self.public_key:
                submission_attrs["base64RsaPublicKey"] = self.public_key
            submission_node = node("submission", method="form-data-post",
                                   **submission_attrs)
            model_children.insert(0, submission_node)

        return node("model",  *model_children)

    def xml_instance(self):
        result = Section.xml_instance(self)

        # set these first to prevent overwriting id and version
        for key, value in self.attribute.items():
            result.setAttribute(unicode(key), value)

        result.setAttribute(u"id", self.id_string)

        # add instance xmlns attribute to the instance node
        if self.instance_xmlns:
            result.setAttribute(u"xmlns", self.instance_xmlns)

        if self.version:
            result.setAttribute(u"version", self.version)

        return result

    def _add_to_nested_dict(self, dicty, path, value):
        if len(path) == 1:
            dicty[path[0]] = value
            return
        if path[0] not in dicty:
            dicty[path[0]] = {}
        self._add_to_nested_dict(dicty[path[0]], path[1:], value)

    def _setup_translations(self):
        """
        set up the self._translations dict which will be referenced in the
        setup media and itext functions
        """
        self._translations = defaultdict(dict)
        for element in self.iter_descendants():
            for d in element.get_translations(self.default_language):
                self._translations[d['lang']][d['path']] = {"long": d['text']}

        # This code sets up translations for choices in filtered selects.
        for list_name, choice_list in self.choices.items():
            for idx, choice in zip(range(len(choice_list)), choice_list):
                for choicePropertyName, choicePropertyValue in choice.items():
                    itext_id = '-'.join(
                        ['static_instance', list_name, str(idx)])
                    if isinstance(choicePropertyValue, dict):
                        for mediatypeorlanguage, value in choicePropertyValue.items():  # noqa
                            if isinstance(value, dict):
                                for language, val in value.items():
                                    self._add_to_nested_dict(
                                        self._translations,
                                        [language, itext_id,
                                         mediatypeorlanguage],
                                        val)
                            else:
                                if choicePropertyName == 'media':
                                    self._add_to_nested_dict(
                                        self._translations,
                                        [self.default_language, itext_id,
                                         mediatypeorlanguage],
                                        value)
                                else:
                                    self._add_to_nested_dict(
                                        self._translations,
                                        [mediatypeorlanguage, itext_id,
                                         'long'], value)
                    elif choicePropertyName == 'label':
                        self._add_to_nested_dict(
                            self._translations,
                            [self.default_language, itext_id, 'long'],
                            choicePropertyValue)

    def _add_empty_translations(self):
        """
        Adds translations so that every itext element has the same elements \
        accross every language.
        When translations are not provided "-" will be used.
        This disables any of the default_language fallback functionality.
        """
        paths = {}
        for lang, translation in self._translations.items():
            for path, content in translation.items():
                paths[path] = paths.get(path, set()).union(content.keys())

        for lang, translation in self._translations.items():
            for path, content_types in paths.items():
                if path not in self._translations[lang]:
                    self._translations[lang][path] = {}
                for content_type in content_types:
                    if content_type not in self._translations[lang][path]:
                        self._translations[lang][path][content_type] = u"-"

    def _setup_media(self):
        """
        Traverse the survey, find all the media, and put in into the \
        _translations data structure which looks like this:
        {language : {element_xpath : {media_type : media}}}
        It matches the xform nesting order.
        """
        if not self._translations:
            self._translations = defaultdict(dict)

        for survey_element in self.iter_descendants():

            translation_key = survey_element.get_xpath() + ":label"
            media_dict = survey_element.get(u"media")

            # This is probably papering over a real problem, but anyway,
            # in py3, sometimes if an item is on an xform with multiple
            # languages and the item only has media defined in # "default"
            # (e.g. no "image" vs. "image::lang"), the media dict will be
            # nested inside of a dict with key "default", e.g.
            # {"default": {"image": "my_image.jpg"}}
            media_dict_default = media_dict.get("default", None)
            if isinstance(media_dict_default, dict):
                media_dict = media_dict_default

            for media_type, possibly_localized_media in media_dict.items():

                if media_type not in SurveyElement.SUPPORTED_MEDIA:
                    raise PyXFormError(
                        "Media type: " + media_type + " not supported")

                if type(possibly_localized_media) is dict:
                    # media is localized
                    localized_media = possibly_localized_media
                else:
                    # media is not localized so create a localized version
                    # using the default language
                    localized_media = {
                        self.default_language: possibly_localized_media
                    }

                for language, media in localized_media.items():

                    # Create the required dictionaries in _translations,
                    # then add media as a leaf value:

                    if language not in self._translations:
                        self._translations[language] = {}

                    translations_language = self._translations[language]

                    if translation_key not in translations_language:
                        translations_language[translation_key] = {}

                    translations_trans_key = \
                        translations_language[translation_key]

                    if media_type not in translations_trans_key:
                            translations_trans_key[media_type] = {}

                    translations_trans_key[media_type] = media

    def itext(self):
        """
        This function creates the survey's itext nodes from _translations
        @see _setup_media _setup_translations
        itext nodes are localized images/audio/video/text
        @see http://code.google.com/p/opendatakit/wiki/XFormDesignGuidelines
        """
        result = []
        for lang, translation in self._translations.items():
            if lang == self.default_language:
                result.append(
                    node("translation", lang=lang, default=u"true()"))
            else:
                result.append(node("translation", lang=lang))

            for label_name, content in translation.items():
                itext_nodes = []
                label_type = label_name.partition(":")[-1]

                if type(content) is not dict:
                    raise Exception()

                for media_type, media_value in content.items():

                    # There is a odk/jr bug where hints can't have a value
                    # for the "form" attribute.
                    # This is my workaround.
                    if label_type == u"hint":
                        value, output_inserted = \
                            self.insert_output_values(media_value)
                        itext_nodes.append(
                            node("value", value, toParseString=output_inserted)
                        )
                        continue

                    if media_type == "long":
                        value, output_inserted = \
                            self.insert_output_values(media_value)
                        # I'm ignoring long types for now because I don't know
                        # how they are supposed to work.
                        itext_nodes.append(
                            node("value", value, toParseString=output_inserted)
                        )
                    elif media_type == "image":
                        value, output_inserted = \
                            self.insert_output_values(media_value)
                        itext_nodes.append(
                            node("value", "jr://images/" + value,
                                 form=media_type,
                                 toParseString=output_inserted)
                        )
                    else:
                        value, output_inserted = \
                            self.insert_output_values(media_value)
                        itext_nodes.append(
                            node("value", "jr://" + media_type + "/" + value,
                                 form=media_type,
                                 toParseString=output_inserted))

                result[-1].appendChild(
                    node("text", *itext_nodes, id=label_name))

        return node("itext", *result)

    def date_stamp(self):
        return self._created.strftime("%Y_%m_%d")

    def _to_ugly_xml(self):
        return '<?xml version="1.0"?>' + self.xml().toxml()

    def _to_pretty_xml(self):
        """
        I want the to_xml method to by default validate the xml we are
        producing.
        """
        # Hacky way of pretty printing xml without adding extra white
        # space to text
        # TODO: check out pyxml
        # http://ronrothman.com/public/leftbraned/xml-dom-minidom-toprettyxml-and-silly-whitespace/
        xml_with_linebreaks = self.xml().toprettyxml(indent='  ')
        text_re = re.compile('(>)\n\s*(\s[^<>\s].*?)\n\s*(\s</)', re.DOTALL)
        output_re = re.compile('\n.*(<output.*>)\n(  )*')
        pretty_xml = text_re.sub(lambda m: ''.join(m.group(1, 2, 3)), xml_with_linebreaks)
        inline_output = output_re.sub('\g<1>', pretty_xml)
        inline_output = re.compile('<label>\s*\n*\s*\n*\s*</label>')\
            .sub('<label></label>', inline_output)
        return '<?xml version="1.0"?>\n' + inline_output

    def __repr__(self):
        return unicode(self)

    def __unicode__(self):
        return "<pyxform.survey.Survey instance at %s>" % hex(id(self))

    def _setup_xpath_dictionary(self):
        self._xpath = {}
        for element in self.iter_descendants():
            if isinstance(element, Question) or isinstance(element, Section):
                if element.name in self._xpath:
                    self._xpath[element.name] = None
                else:
                    self._xpath[element.name] = element.get_xpath()

    def _var_repl_function(self, matchobj):
        """
        Given a dictionary of xpaths, return a function we can use to
        replace ${varname} with the xpath to varname.
        """
        name = matchobj.group(1)
        intro = "There has been a problem trying to replace ${%s} with the "\
            "XPath to the survey element named '%s'." % (name, name)
        if name not in self._xpath:
            raise PyXFormError(
                intro + " There is no survey element with this name.")
        if self._xpath[name] is None:
            raise PyXFormError(intro + " There are multiple survey elements"
                               " with this name.")

        return " " + self._xpath[name] + " "

    def insert_xpaths(self, text):
        """
        Replace all instances of ${var} with the xpath to var.
        """
        bracketed_tag = r"\$\{(.*?)\}"

        return re.sub(bracketed_tag, self._var_repl_function, unicode(text))

    def _var_repl_output_function(self, matchobj):
        """
        A regex substitution function that will replace
        ${varname} with an output element that has the xpath to varname.
        """
#        if matchobj.group(1) not in self._xpath:
#            raise PyXFormError("There is no survey element with this name.",
#                            matchobj.group(1))
        return '<output value="' + self._var_repl_function(matchobj) + '" />'

    def insert_output_values(self, text):
        """
        Replace all the ${variables} in text with xpaths.
        Returns that and a boolean indicating if there were any ${variables}
        present.
        """
        # There was a bug where escaping is completely turned off in labels
        # where variable replacement is used.
        # For exampke, `${name} < 3` causes an error but `< 3` does not.
        # This is my hacky fix for it, which does string escaping prior to
        # variable replacement:
        text_node = PatchedText()
        text_node.data = text
        xml_text = text_node.toxml()

        bracketed_tag = r"\$\{(.*?)\}"
        # need to make sure we have reason to replace
        # since at this point < is &lt,
        # the net effect &lt gets translated again to &amp;lt;
        if unicode(xml_text).find('{') != -1:
            result = re.sub(
                bracketed_tag, self._var_repl_output_function,
                unicode(xml_text))
            return result, not result == xml_text
        return text, False

    def print_xform_to_file(self, path=None, validate=True, pretty_print=True, warnings=None):
        """
        Print the xForm to a file and optionally validate it as well by
        throwing exceptions and adding warnings to the warnings array.
        """
        if warnings is None:
            warnings = []
        if not path:
            path = self._print_name + ".xml"
        try:
            with codecs.open(path, mode="w", encoding="utf-8") as fp:
                if pretty_print:
                    fp.write(self._to_pretty_xml())
                else:
                    fp.write(self._to_ugly_xml())
        except Exception as e:
            if os.path.exists(path):
                os.unlink(path)
            raise e
        if validate:
            warnings.extend(check_xform(path))

    def to_xml(self, validate=True, pretty_print=True, warnings=None):
        # On Windows, NamedTemporaryFile must be opened exclusively.
        # So it must be explicitly created, opened, closed, and removed.
        tmp = tempfile.NamedTemporaryFile(delete=False)
        tmp.close()
        try:
            # this will throw an exception if the xml is not valid
            self.print_xform_to_file(tmp.name, validate=validate,
                                     pretty_print=pretty_print,
                                     warnings=warnings)
        finally:
            if os.path.exists(tmp.name):
                os.remove(tmp.name)
        if pretty_print:
            return self._to_pretty_xml()
        else:
            return self._to_ugly_xml()

    def instantiate(self):
        """
        Instantiate as in return a instance of SurveyInstance for collected
        data.
        """
        return SurveyInstance(self)



from __future__ import print_function
import re
import json
import copy
import codecs
import xml.etree.ElementTree as ETree
from operator import itemgetter
from pyxform import builder
from pyxform.utils import basestring
from pyxform.survey import nsmap


# {{{ http://code.activestate.com/recipes/573463/ (r7)
class XmlDictObject(dict):
    """
    Adds object like functionality to the standard dictionary.
    """

    def __init__(self, initdict=None):
        if initdict is None:
            initdict = {}
        dict.__init__(self, initdict)

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __setattr__(self, item, value):
        self.__setitem__(item, value)

    def __str__(self):
        if '_text' in self:
            return self.__getitem__('_text')
        else:
            return ''

    @staticmethod
    def wrap(x):
        """
        Static method to wrap a dictionary recursively as an XmlDictObject
        """

        if isinstance(x, dict):
            return XmlDictObject(
                (k, XmlDictObject.Wrap(v)) for (k, v) in iter(x.items()))
        elif isinstance(x, list):
            return [XmlDictObject.Wrap(v) for v in x]
        else:
            return x

    @staticmethod
    def _un_wrap(x):
        if isinstance(x, dict):
            return dict(
                (k, XmlDictObject._un_wrap(v)) for (k, v) in iter(x.items()))
        elif isinstance(x, list):
            return [XmlDictObject._un_wrap(v) for v in x]
        else:
            return x

    def un_wrap(self):
        """
        Recursively converts an XmlDictObject to a standard dictionary
        and returns the result.
        """

        return XmlDictObject._un_wrap(self)


def _convert_dict_to_xml_recurse(parent, dictitem):
    assert not isinstance(dictitem, list)

    if isinstance(dictitem, dict):
        for (tag, child) in iter(dictitem.items()):
            if str(tag) == '_text':
                parent.text = str(child)
            elif isinstance(child, list):
                # iterate through the array and convert
                for listchild in child:
                    elem = ETree.Element(tag)
                    parent.append(elem)
                    _convert_dict_to_xml_recurse(elem, listchild)
            else:
                elem = ETree.Element(tag)
                parent.append(elem)
                _convert_dict_to_xml_recurse(elem, child)
    else:
        parent.text = str(dictitem)


def convert_dict_to_xml(xmldict):
    """
    Converts a dictionary to an XML ElementTree Element
    """

    roottag = xmldict.keys()[0]
    root = ETree.Element(roottag)
    _convert_dict_to_xml_recurse(root, xmldict[roottag])
    return root


def _convert_xml_to_dict_recurse(node, dictclass):
    nodedict = dictclass()

    if len(node.items()) > 0:
        # if we have attributes, set them
        nodedict.update(dict(node.items()))

    for child in node:
        # recursively add the element's children
        newitem = _convert_xml_to_dict_recurse(child, dictclass)
        # if tag in between text node, capture the tail end
        if child.tail is not None and child.tail.strip() != '':
            newitem['tail'] = child.tail
        if child.tag in nodedict:
            # found duplicate tag, force a list
            if isinstance(nodedict[child.tag], list):
                # append to existing list
                nodedict[child.tag].append(newitem)
            else:
                # convert to list
                nodedict[child.tag] = [nodedict[child.tag], newitem]
        else:
            # only one, directly set the dictionary
            nodedict[child.tag] = newitem

    if node.text is None:
        text = ''
    else:
        text = node.text.strip()

    if len(nodedict) > 0:
        # if we have a dictionary
        # add the text as a dictionary value (if there is any)
        if len(text) > 0:
            nodedict['_text'] = text
    else:
        # if we don't have child nodes or attributes, just set the text
        nodedict = text

    return nodedict


def convert_xml_to_dict(root, dictclass=XmlDictObject):
    """
    Converts an XML file or ElementTree Element to a dictionary
    """
    # If a string is passed in, try to open it as a file
    if isinstance(root, basestring):
        root = _try_parse(root)
    elif not isinstance(root, ETree.Element):
        raise TypeError('Expected ElementTree.Element or file path string')

    return dictclass({root.tag: _convert_xml_to_dict_recurse(root, dictclass)})


# end of http://code.activestate.com/recipes/573463/ }}}


def _try_parse(root, parser=None):
    """
    Try to parse the root from a string or a file/file-like object.
    """
    root = root.encode("UTF-8")
    try:
        parsed_root = ETree.fromstring(root, parser)
    except ETree.ParseError:
        parsed_root = ETree.parse(root, parser=parser).getroot()
    return parsed_root


class XFormToDict:
    def __init__(self, root):
        if isinstance(root, basestring):
            parser = ETree.XMLParser(encoding="UTF-8")
            self._root = _try_parse(root, parser)
            self._dict = XmlDictObject(
                {self._root.tag: _convert_xml_to_dict_recurse(
                    self._root, XmlDictObject)})
        elif not isinstance(root, ETree.Element):
            raise TypeError('Expected ElementTree.Element or file path string')

    def get_dict(self):
        json_str = json.dumps(self._dict)
        for uri in nsmap.values():
            json_str = json_str.replace('{%s}' % uri, '')
        return json.loads(json_str)


def create_survey_element_from_xml(xml_file):
    sb = XFormToDictBuilder(xml_file)
    return sb.survey()


class XFormToDictBuilder:
    """Experimental XFORM xml to XFORM JSON"""
    QUESTION_TYPES = {
        'select': 'select all that apply',
        'select1': 'select one',
        'int': 'integer',
        'dateTime': 'datetime',
        'string': 'text'
    }

    def __init__(self, xml_file):
        doc_as_dict = XFormToDict(xml_file).get_dict()
        self._xmldict = doc_as_dict

        assert 'html' in doc_as_dict
        assert 'body' in doc_as_dict['html']
        assert 'head' in doc_as_dict['html']
        assert 'model' in doc_as_dict['html']['head']
        assert 'title' in doc_as_dict['html']['head']
        assert 'bind' in doc_as_dict['html']['head']['model']

        self.body = doc_as_dict['html']['body']
        self.model = doc_as_dict['html']['head']['model']
        self.bindings = copy.deepcopy(self.model['bind'])
        self._bind_list = copy.deepcopy(self.model['bind'])
        self.title = doc_as_dict['html']['head']['title']
        self.new_doc = {
            "type": "survey",
            "title": self.title,
            "children": [],
            "id_string": self.title,
            "sms_keyword": self.title,
            "default_language": "default",
        }
        self._set_submission_info()
        self._set_survey_name()
        self.children = []
        self.ordered_binding_refs = []
        self._set_binding_order()

        # set self.translations
        self._set_translations()

        for key, obj in iter(self.body.items()):
            if isinstance(obj, dict):
                self.children.append(
                    self._get_question_from_object(obj, type=key))
            elif isinstance(obj, list):
                for item in obj:
                    self.children.append(
                        self._get_question_from_object(item, type=key))
        self._cleanup_bind_list()
        self._cleanup_children()
        self.new_doc['children'] = self.children

    def _set_binding_order(self):
        self.ordered_binding_refs = []
        for bind in self.bindings:
            self.ordered_binding_refs.append(bind['nodeset'])

    def _set_survey_name(self):
        obj = self.bindings[0]
        name = obj['nodeset'].split('/')[1]
        self.new_doc['name'] = name
        self.new_doc['id_string'] = self.model['instance'][name]['id']

    def _set_submission_info(self):
        if 'submission' in self.model:
            submission = self.model['submission']
            if 'action' in submission:
                self.new_doc['submission_url'] = submission['action']
            if 'base64RsaPublicKey' in submission:
                self.new_doc['public_key'] = submission['base64RsaPublicKey']

    def _cleanup_children(self):
        def remove_refs(children):
            for child in children:
                if isinstance(child, dict):
                    if 'nodeset' in child:
                        del child['nodeset']
                    if 'ref' in child:
                        del child['ref']
                    if '__order' in child:
                        del child['__order']
                    if 'children' in child:
                        remove_refs(child['children'])

        # do some ordering, order is specified by bindings
        def order_children(children):
            if isinstance(children, list):
                try:
                    children.sort(key=itemgetter('__order'))
                except KeyError:
                    pass
                for child in children:
                    if isinstance(child, dict) and 'children' in child:
                        order_children(child['children'])

        order_children(self.children)
        remove_refs(self.children)

    def _cleanup_bind_list(self):
        for item in self._bind_list:
            ref = item['nodeset']
            name = self._get_name_from_ref(ref)
            parent_ref = ref[:ref.find('/%s' % name)]
            question = self._get_question_params_from_bindings(ref)
            question['name'] = name
            question['__order'] = self._get_question_order(ref)
            if 'calculate' in item:
                question['type'] = 'calculate'
            if ref.split('/').__len__() == 3:
                # just append on root node, has no group
                question['ref'] = ref
                self.children.append(question)
                continue
            for child in self.children:
                if child['ref'] == parent_ref:
                    question['ref'] = ref
                    updated = False
                    for c in child['children']:
                        if isinstance(c, dict) \
                                and 'ref' in c and c['ref'] == ref:
                            c.update(question)
                            updated = True
                    if not updated:
                        child['children'].append(question)
            if 'ref' not in question:
                new_ref = u'/'.join(ref.split('/')[2:])
                root_ref = u'/'.join(ref.split('/')[:2])
                q = self._get_item_func(root_ref, new_ref, item)
                if 'type' not in q and 'type' in question:
                    q.update(question)
                if q['type'] == 'group' and q['name'] == 'meta':
                    q['control'] = {'bodyless': True}
                    q['__order'] = self._get_question_order(ref)
                self.children.append(q)
                self._bind_list.append(item)
                break
        if self._bind_list:
            self._cleanup_bind_list()

    def _get_item_func(self, ref, name, item):
        rs = {}
        name_splits = name.split('/')
        rs['name'] = name_splits[0]
        ref = '%s/%s' % (ref, rs['name'])
        rs['ref'] = ref
        if name_splits.__len__() > 1:
            rs['type'] = 'group'
            rs['children'] = [
                self._get_item_func(ref, '/'.join(name_splits[1:]), item)]
        return rs

    def survey(self):
        new_doc = json.dumps(self.new_doc)
        _survey = builder.create_survey_element_from_json(new_doc)
        return _survey

    def _get_question_order(self, ref):
        try:
            return self.ordered_binding_refs.index(ref)
        except ValueError:
            # likely a group
            for i in self.ordered_binding_refs:
                if i.startswith(ref):
                    return self.ordered_binding_refs.index(i) + 1
            return self.ordered_binding_refs.__len__() + 1

    def _get_question_from_object(self, obj, type=None):
        try:
            ref = obj['ref']
        except KeyError:
            try:
                ref = obj['nodeset']
            except KeyError:
                raise TypeError(
                    'cannot find "ref" or "nodeset" in {}'.format(repr(obj)))
        question = {'ref': ref, '__order': self._get_question_order(ref),
                    'name': self._get_name_from_ref(ref)}
        if 'hint' in obj:
            k, v = self._get_label(obj['hint'], 'hint')
            question[k] = v
        if 'label' in obj:
            k, v = self._get_label(obj['label'])
            if isinstance(v, dict) and 'label' in v.keys() \
                    and 'media' in v.keys():
                for _k, _v in iter(v.items()):
                    question[_k] = _v
            else:
                question[k] = v
        if 'autoplay' in obj or 'appearance' in obj \
                or 'count' in obj or 'rows' in obj:
            question['control'] = {}
        if 'appearance' in obj:
            question["control"].update({'appearance': obj['appearance']})
        if 'rows' in obj:
            question['control'].update({'rows': obj['rows']})
        if 'autoplay' in obj:
            question['control'].update({'autoplay': obj['autoplay']})
        question_params = self._get_question_params_from_bindings(ref)
        if isinstance(question_params, dict):
            for k, v in iter(question_params.items()):
                question[k] = v
        # has to come after the above block
        if 'mediatype' in obj:
            question['type'] = obj['mediatype'].replace('/*', '')
        if 'item' in obj:
            children = []
            for i in obj['item']:
                if isinstance(i, dict) and \
                                'label' in i.keys() and 'value' in i.keys():
                    k, v = self._get_label(i['label'])
                    children.append(
                        {'name': i['value'], k: v})
            question['children'] = children
        question_type = question['type'] if 'type' in question else type
        if question_type == 'text' and 'bind' in question \
                and 'readonly' in question['bind']:
            question_type = question['type'] = 'note'
            del question['bind']['readonly']
            if len(question['bind'].keys()) == 0:
                del question['bind']
        if question_type in ['group', 'repeat']:
            if question_type == 'group' and 'repeat' in obj:
                question['children'] = \
                    self._get_children_questions(obj['repeat'])
                question_type = 'repeat'
                if 'count' in obj['repeat']:
                    if 'control' not in question:
                        question['control'] = {}
                    question['control'].update(
                        {'jr:count':
                            self._shorten_xpaths_in_string(
                                obj['repeat']['count'].strip())})
            else:
                question['children'] = self._get_children_questions(obj)
            question['type'] = question_type
        if type == 'trigger':
            question['type'] = 'acknowledge'
        if question_type == 'geopoint' and 'hint' in question:
            del question['hint']
        if 'type' not in question and type:
            question['type'] = question_type
        return question

    def _get_children_questions(self, obj):
        children = []
        for k, v in iter(obj.items()):
            if k in ['ref', 'label', 'nodeset']:
                continue
            if isinstance(v, dict):
                child = self._get_question_from_object(v, type=k)
                children.append(child)
            elif isinstance(v, list):
                for i in v:
                    child = self._get_question_from_object(i, type=k)
                    children.append(child)
        return children

    def _get_question_params_from_bindings(self, ref):
        for item in self.bindings:
            if item['nodeset'] == ref:
                try:
                    self._bind_list.remove(item)
                except ValueError:
                    pass
                rs = {}
                for k, v in iter(item.items()):
                    if k == 'nodeset':
                        continue
                    if k == 'type':
                        v = self._get_question_type(v)
                    if k in ['relevant', 'required', 'constraint',
                             'constraintMsg', 'readonly', 'calculate',
                             'noAppErrorString', 'requiredMsg']:
                        if k == 'noAppErrorString':
                            k = 'jr:noAppErrorString'
                        if k == 'requiredMsg':
                            k = 'jr:requiredMsg'
                        if k == 'constraintMsg':
                            k = "jr:constraintMsg"
                            v = self._get_constraint_msg(v)
                        if k == 'required':
                            if v == 'true()':
                                v = 'yes'
                            elif v == 'false()':
                                v = 'no'
                        if k in ['constraint', 'relevant', 'calculate']:
                            v = self._shorten_xpaths_in_string(v)
                        if 'bind' not in rs:
                            rs['bind'] = {}
                        rs['bind'][k] = v
                        continue
                    rs[k] = v
                if 'preloadParams' in rs and 'preload' in rs:
                    rs['type'] = rs['preloadParams']
                    del rs['preloadParams']
                    del rs['preload']
                return rs
        return None

    def _get_question_type(self, type):
        if type in self.QUESTION_TYPES.keys():
            return self.QUESTION_TYPES[type]
        return type

    def _set_translations(self):
        if 'itext' not in self.model:
            self.translations = []
            return
        assert 'translation' in self.model['itext']
        self.translations = self.model['itext']['translation']
        if isinstance(self.translations, dict):
            self.translations = [self.translations]
        assert 'text' in self.translations[0]
        assert 'lang' in self.translations[0]

    def _get_label(self, label_obj, key='label'):
        if isinstance(label_obj, dict):
            try:
                ref = label_obj['ref'].replace(
                    'jr:itext(\'', '').replace('\')', '')
            except KeyError:
                return key, self._get_output_text(label_obj)
            else:
                return self._get_text_from_translation(ref, key)
        return key, label_obj

    def _get_output_text(self, value):
        if 'output' in value and '_text' in value:
            v = [value['_text'], self._get_bracketed_name(
                value['output']['value'])]
            text = u' '.join(v)
            if 'tail' in value['output']:
                text = u''.join(
                    [text, value['output']['tail']])
        elif 'output' in value and '_text' not in value:
            text = self._get_bracketed_name(
                value['output']['value'])
        else:
            return value
        return text

    def _get_text_from_translation(self, ref, key='label'):
        label = {}
        for translation in self.translations:
            lang = translation['lang']
            label_list = translation['text']
            for l in label_list:
                if l['value'] == '-':  # skip blank label
                    continue
                if l['id'] == ref:
                    text = value = l['value']
                    if isinstance(value, dict):
                        if 'output' in value:
                            text = self._get_output_text(value)
                        if 'form' in value and '_text' in value:
                            key = u'media'
                            v = value['_text']
                            if value['form'] == 'image':
                                v = v.replace('jr://images/', '')
                            else:
                                v = v.replace('jr://%s/' % value['form'], '')
                            if v == '-':  # skip blank
                                continue
                            text = {value['form']: v}
                    if isinstance(value, list):
                        for item in value:
                            if 'form' in item and '_text' in item:
                                k = u'media'
                                m_type = item['form']
                                v = item['_text']
                                if m_type == 'image':
                                    v = v.replace('jr://images/', '')
                                else:
                                    v = v.replace('jr://%s/' % m_type, '')
                                if v == '-':
                                    continue
                                if k not in label:
                                    label[k] = {}
                                if m_type not in label[k]:
                                    label[k][m_type] = {}
                                label[k][m_type][lang] = v
                                continue
                            if isinstance(item, basestring):
                                if item == '-':
                                    continue
                            if 'label' not in label:
                                label['label'] = {}
                            label['label'][lang] = item
                        continue

                    label[lang] = text
                    break
        if key == u'media' and label.keys() == ['default']:
            label = label['default']
        return key, label

    def _get_bracketed_name(self, ref):
        name = self._get_name_from_ref(ref)
        return u''.join([u'${', name.strip(), u'}'])

    def _get_constraint_msg(self, constraint_msg):
        if isinstance(constraint_msg, basestring):
            if constraint_msg.find(':jr:constraintMsg') != -1:
                ref = constraint_msg.replace(
                    'jr:itext(\'', '').replace('\')', '')
                k, constraint_msg = self._get_text_from_translation(ref)
        return constraint_msg

    def _get_name_from_ref(self, ref):
        """given /xlsform_spec_test/launch,
        return the string after the last occurance of the character '/'
        """
        pos = ref.rfind('/')
        if pos == -1:
            return ref
        else:
            return ref[pos + 1:].strip()

    def _expand_child(self, obj_list):
        return obj_list

    def _shorten_xpaths_in_string(self, text):
        def get_last_item(xpath_str):
            l = xpath_str.split("/")
            return l[len(l) - 1].strip()

        def replace_function(match):
            return "${%s}" % get_last_item(match.group())

        # moving re flags into compile for python 2.6 compat
        pattern = "( /[a-z0-9\-_]+(?:/[a-z0-9\-_]+)+ )"
        text = re.compile(pattern, flags=re.I).sub(replace_function, text)
        pattern = "(/[a-z0-9\-_]+(?:/[a-z0-9\-_]+)+)"
        text = re.compile(pattern, flags=re.I).sub(replace_function, text)
        return text


def write_object_to_file(filename, obj):
    f = codecs.open(filename, 'w', encoding='utf-8')
    f.write(json.dumps(obj, indent=2))
    f.close()
    print("object written to file: ", filename)



"""
Some tests for the new (v0.9) spec is properly implemented.  
"""
import unittest2 as unittest
import codecs
import os
import pyxform
from pyxform.tests.utils import XFormTestCase

DIR = os.path.dirname(__file__)


class MainTest(XFormTestCase):
    maxDiff = None

    def runTest(self):
        filename = "or_other.xlsx"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        expected_output_path = os.path.join(DIR, "test_expected_output",
                                            root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(path_to_excel_file,
                                                          warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)
        # print warnings
        # Compare with the expected output:
        with codecs.open(expected_output_path, 'rb',
                         encoding="utf-8") as expected_file:
            with codecs.open(output_path, 'rb',
                             encoding="utf-8") as actual_file:
                self.assertXFormEqual(expected_file.read(), actual_file.read())


if __name__ == '__main__':
    unittest.main()



"""
Testing simple cases for pyxform
"""
from unittest import TestCase
from pyxform.survey import Survey

from pyxform.builder import create_survey_element_from_dict


# TODO:
#  * test_two_questions_with_same_id_fails
#     (get this working in json2xform)

class BasicJson2XFormTests(TestCase):
    def test_survey_can_have_to_xml_called_twice(self):
        """
        Test: Survey can have "to_xml" called multiple times
        
        (This was not being allowed before.)
        
        It would be good to know (with confidence) that a survey object
        can be exported to_xml twice, and the same thing will be returned
        both times.
        """
        survey = Survey(name=u"SampleSurvey")
        q = create_survey_element_from_dict(
            {u'type': u'text', u'name': u'name', u'label': u'label'})
        survey.add_child(q)

        str1 = survey.to_xml()
        str2 = survey.to_xml()

        self.assertEqual(str1, str2)



from pyxform.builder import create_survey_from_path
from pyxform.builder import create_survey_element_from_json
from pyxform.tests import utils
import os
import codecs


class TestFileType(utils.XFormTestCase):

    maxDiff = None

    def test_create_from_path(self):
        path = utils.path_to_text_fixture("file_type.xlsx")
        survey = create_survey_from_path(path)
        path = os.path.join(
            os.path.dirname(__file__), 'test_expected_output', 'file_type.xml')

        with codecs.open(path, encoding='utf-8') as f:
            expected_xml = f.read()
            self.assertXFormEqual(survey.to_xml(), expected_xml)

            survey = create_survey_element_from_json(survey.to_json())
            self.assertXFormEqual(survey.to_xml(), expected_xml)



from __future__ import print_function

"""
odk_validate.py
A python wrapper around ODK Validate
"""
import os
import re
import sys
from subprocess import Popen, PIPE
import threading
import signal

CURRENT_DIRECTORY = os.path.dirname(os.path.realpath(__file__))
ODK_VALIDATE_JAR = os.path.join(CURRENT_DIRECTORY, "ODK_Validate.jar")


class ODKValidateError(Exception):
    pass


# Adapted from:
# http://betabug.ch/blogs/ch-athens/1093
def run_popen_with_timeout(command, timeout):
    """
    Run a sub-program in subprocess.Popen, pass it the input_data,
    kill it if the specified timeout has passed.
    returns a tuple of resultcode, timeout, stdout, stderr
    """
    kill_check = threading.Event()

    def _kill_process_after_a_timeout(pid):
        os.kill(pid, signal.SIGTERM)
        kill_check.set()  # tell the main routine that we had to kill
        # use SIGKILL if hard to kill...
        return

    p = Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE)
    pid = p.pid
    watchdog = threading.Timer(
        timeout, _kill_process_after_a_timeout, args=(pid,))
    watchdog.start()
    (stdout, stderr) = p.communicate()
    watchdog.cancel()  # if it's still waiting to run
    timeout = kill_check.isSet()
    kill_check.clear()
    return p.returncode, timeout, stdout, stderr


def _java_installed():
    # This alternative allows for java detection on Windows.
    try:
        p = Popen(["which", "java"], stdout=PIPE).stdout.readlines()
        found = len(p) != 0
    except WindowsError:
        p = Popen('java -version', stderr=PIPE, stdout=PIPE).stderr.read()
        found = p.startswith('java version'.encode())
    return found


def _cleanup_errors(error_message):
    def get_last_item(xpath_str):
        l = xpath_str.split("/")
        return l[len(l) - 1]

    def replace_function(match):
        strmatch = match.group()
        # eliminate e.g /html/body/select1[@ref=/id_string/elId]/item/value
        # instance('q4')/root/item[...]
        if strmatch.startswith("/html/body") \
                or strmatch.startswith("/root/item") \
                or strmatch.startswith("/html/head/model/bind") \
                or strmatch.endswith("/item/value"):
            return strmatch
        return "${%s}" % get_last_item(match.group())

    pattern = "(/[a-z0-9\-_]+(?:/[a-z0-9\-_]+)+)"
    # moving flags into compile for python 2.6 compat
    error_message = re.compile(pattern, flags=re.I).sub(replace_function,
                                                        error_message)
    k = []
    lastline = ''
    for line in error_message.splitlines():
        # has a java filename (with line number)
        has_java_filename = line.find('.java:') is not -1
        # starts with '    at java class path or method path'
        is_a_java_method = line.find('\tat') is not -1
        # is the same as the last line
        is_duplicate = (line == lastline)
        lastline = line
        if not has_java_filename and not is_a_java_method and not is_duplicate:
            # remove java.lang.RuntimeException
            if line.startswith('java.lang.RuntimeException: '):
                line = line.replace('java.lang.RuntimeException: ', '')
            # remove org.javarosa.xpath.XPathUnhandledException
            if line.startswith('org.javarosa.xpath.XPathUnhandledException: '):
                line = line.replace(
                    'org.javarosa.xpath.XPathUnhandledException: ', '')
            # remove java.lang.NullPointerException
            if line.startswith('java.lang.NullPointerException'):
                continue
            k.append(line)
    return u'\n'.join(k)


def check_xform(path_to_xform):
    """
    Returns an array of warnings if the form is valid.
    Throws an exception if it is not
    """
    # provide useful error message if java is not installed
    if not _java_installed():
        raise EnvironmentError(
            "pyxform odk validate dependency: java not found")

    # resultcode indicates validity of the form
    # timeout indicates whether validation ran out of time to complete
    # stdout is not used because it has some warnings that always
    # appear and can be ignored.
    # stderr is treated as a warning if the form is valid or an error
    # if it is invalid.
    returncode, timeout, stdout, stderr = run_popen_with_timeout(
        ["java", "-jar", ODK_VALIDATE_JAR, path_to_xform], 100)
    warnings = []
    # On Windows, stderr may be latin-1; in which case utf-8 decode will fail.
    # If both utf-8 and latin-1 decoding fail then raise all as IOError.
    # If the above validate jar call fails, add make sure that the java path
    #  is set, e.g. PATH=C:\Program Files (x86)\Java\jre1.8.0_71\bin
    try:
        stderr = stderr.decode('utf-8')
    except UnicodeDecodeError as ude:
        try:
            stderr = stderr.decode('latin-1')
        except BaseException as be:
            msg = "Failed to decode validate stderr as utf-8 or latin-1."
            raise IOError(msg, ude, be)

    if timeout:
        return ["XForm took to long to completely validate."]
    else:
        if returncode > 0:  # Error invalid
            raise ODKValidateError(
                'ODK Validate Errors:\n' + _cleanup_errors(stderr))
        elif returncode == 0:
            if stderr:
                warnings.append('ODK Validate Warnings:\n' + stderr)
            return warnings
        elif returncode < 0:
            return ["Bad return code from ODK Validate."]


if __name__ == '__main__':
    print(__doc__)
    check_xform(sys.argv[1])



"""
Some tests for the new (v0.9) spec is properly implemented.
"""
import unittest2 as unittest
import codecs
import os
import pyxform
from pyxform.utils import has_external_choices
from pyxform.xls2json import SurveyReader
from pyxform.tests.utils import XFormTestCase
from pyxform.errors import PyXFormError

DIR = os.path.dirname(__file__)


class GroupNames(unittest.TestCase):
    maxDiff = None

    def runTest(self):
        filename = "group_name_test.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        with self.assertRaises(Exception):
            json_survey = pyxform.xls2json.parse_file_to_json(
                path_to_excel_file, warnings=warnings)
            survey = pyxform.create_survey_element_from_dict(json_survey)
            survey.print_xform_to_file(output_path, warnings=warnings)


class DuplicateColumns(unittest.TestCase):
    maxDiff = None

    def runTest(self):
        filename = "duplicate_columns.xlsx"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        with self.assertRaises(Exception):
            json_survey = pyxform.xls2json.parse_file_to_json(
                path_to_excel_file, warnings=warnings)
            survey = pyxform.create_survey_element_from_dict(json_survey)
            survey.print_xform_to_file(output_path, warnings=warnings)


class RepeatDateTest(XFormTestCase):
    maxDiff = None

    def runTest(self):
        filename = "repeat_date_test.xls"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        expected_output_path = os.path.join(
            DIR, "test_expected_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(
            path_to_excel_file, warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)
        # print warnings
        # Compare with the expected output:
        with codecs.open(
                expected_output_path, 'rb', encoding="utf-8") as expected_file:
            with codecs.open(
                    output_path, 'rb', encoding="utf-8") as actual_file:
                self.assertXFormEqual(expected_file.read(), actual_file.read())


class XmlEscaping(XFormTestCase):
    maxDiff = None

    def runTest(self):
        filename = "xml_escaping.xls"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        expected_output_path = os.path.join(
            DIR, "test_expected_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(
            path_to_excel_file, warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)
        # print warnings
        # Compare with the expected output:
        with codecs.open(
                expected_output_path, 'rb', encoding="utf-8") as expected_file:
            with codecs.open(
                    output_path, 'rb', encoding="utf-8") as actual_file:
                self.assertXFormEqual(expected_file.read(), actual_file.read())


class DefaultTimeTest(XFormTestCase):
    maxDiff = None

    def runTest(self):
        filename = "default_time_demo.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        expected_output_path = os.path.join(
            DIR, "test_expected_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(
            path_to_excel_file, warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)
        # print warnings
        # Compare with the expected output:
        with codecs.open(
                expected_output_path, 'rb', encoding="utf-8") as expected_file:
            with codecs.open(
                    output_path, 'rb', encoding="utf-8") as actual_file:
                self.assertXFormEqual(expected_file.read(), actual_file.read())


class CascadeOldFormat(unittest.TestCase):
    maxDiff = None

    def runTest(self):
        filename = "cascades_old.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(
            path_to_excel_file, warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)


class ValidateWrapper(unittest.TestCase):
    maxDiff = None

    def runTest(self):
        filename = "ODKValidateWarnings.xlsx"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        json_survey = pyxform.xls2json.parse_file_to_json(
            path_to_excel_file, warnings=warnings)
        survey = pyxform.create_survey_element_from_dict(json_survey)
        survey.print_xform_to_file(output_path, warnings=warnings)


class CascadeOldFormatIndexError(unittest.TestCase):
    maxDiff = None

    def runTest(self):
        filename = "cascades_old_with_no_cascade_sheet.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        # Get the xform output path:
        root_filename, ext = os.path.splitext(filename)
        output_path = os.path.join(DIR, "test_output", root_filename + ".xml")
        # Do the conversion:
        warnings = []
        with self.assertRaises(PyXFormError):
            json_survey = pyxform.xls2json.parse_file_to_json(
                path_to_excel_file, warnings=warnings)
            survey = pyxform.create_survey_element_from_dict(json_survey)
            survey.print_xform_to_file(output_path, warnings=warnings)


class EmptyStringOnRelevantColumnTest(unittest.TestCase):
    def runTest(self):
        filename = "ict_survey_fails.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        workbook_dict = pyxform.xls2json.parse_file_to_workbook_dict(
            path_to_excel_file)
        with self.assertRaises(KeyError):
            # bind:relevant should not be part of workbook_dict
            workbook_dict['survey'][0][u'bind: relevant'].strip()


class BadChoicesSheetHeaders(unittest.TestCase):
    def runTest(self):
        filename = "spaces_in_choices_header.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        warnings = []
        pyxform.xls2json.parse_file_to_json(path_to_excel_file,
                                            warnings=warnings)
        self.assertEquals(len(warnings), 2,
                          "Found " + str(len(warnings)) + " warnings")


class TestChoiceNameAsType(unittest.TestCase):
    def test_choice_name_as_type(self):
        filename = "choice_name_as_type.xls"
        path_to_excel_file = os.path.join(DIR, "example_xls", filename)
        xls_reader = SurveyReader(path_to_excel_file)
        survey_dict = xls_reader.to_json_dict()
        self.assertTrue(has_external_choices(survey_dict))


class TestBlankSecondRow(unittest.TestCase):
    def test_blank_second_row(self):
        filename = "blank_second_row.xls"
        path_to_excel_file = os.path.join(DIR, "bug_example_xls", filename)
        xls_reader = SurveyReader(path_to_excel_file)
        survey_dict = xls_reader.to_json_dict()
        self.assertTrue(len(survey_dict) > 0)


if __name__ == '__main__':
    unittest.main()



import os.path

from pyxform.errors import PyXFormError
from pyxform.question_type_dictionary import QUESTION_TYPE_DICT
from pyxform.survey_element import SurveyElement
from pyxform.utils import basestring, node, unicode


class Question(SurveyElement):

    def validate(self):
        SurveyElement.validate(self)

        # make sure that the type of this question exists in the
        # question type dictionary.
        if self.type not in QUESTION_TYPE_DICT:
            raise PyXFormError(
                "Unknown question type '%s'." % self.type
                )

    def xml_instance(self):
        survey = self.get_root()
        attributes = {}
        attributes.update(self.get(u'instance', {}))
        for key, value in attributes.items():
            attributes[key] = survey.insert_xpaths(value)
        if self.get(u"default"):
            return node(
                self.name, unicode(self.get(u"default")), **attributes
            )
        return node(self.name, **attributes)

    def xml_control(self):
        return None


class InputQuestion(Question):
    """
    This control string is the same for: strings, integers, decimals,
    dates, geopoints, barcodes ...
    """
    def xml_control(self):
        control_dict = self.control
        label_and_hint = self.xml_label_and_hint()
        survey = self.get_root()
        # Resolve field references in attributes
        for key, value in control_dict.items():
            control_dict[key] = survey.insert_xpaths(value)
        control_dict['ref'] = self.get_xpath()

        result = node(**control_dict)
        if label_and_hint:
            for element in self.xml_label_and_hint():
                result.appendChild(element)

        # Input types are used for selects with external choices sheets.
        if self['query']:
            choice_filter = self.get('choice_filter')
            query = "instance('" + self['query'] + "')/root/item"
            choice_filter = survey.insert_xpaths(choice_filter)
            if choice_filter:
                query += '[' + choice_filter + ']'
            result.setAttribute('query', query)
        return result


class TriggerQuestion(Question):

    def xml_control(self):
        control_dict = self.control
        survey = self.get_root()
        # Resolve field references in attributes
        for key, value in control_dict.items():
            control_dict[key] = survey.insert_xpaths(value)
        control_dict['ref'] = self.get_xpath()
        return node(
            u"trigger",
            *self.xml_label_and_hint(),
            **control_dict
            )


class UploadQuestion(Question):
    def _get_media_type(self):
        return self.control[u"mediatype"]

    def xml_control(self):
        control_dict = self.control
        control_dict['ref'] = self.get_xpath()
        control_dict['mediatype'] = self._get_media_type()
        return node(
            u"upload",
            *self.xml_label_and_hint(),
            **control_dict
            )


class Option(SurveyElement):

    def xml_value(self):
        return node(u"value", self.name)

    def xml(self):
        item = node(u"item")
        self.xml_label()
        item.appendChild(self.xml_label())
        item.appendChild(self.xml_value())

        return item

    def validate(self):
        pass


class MultipleChoiceQuestion(Question):

    def __init__(self, **kwargs):
        kwargs_copy = kwargs.copy()
        # Notice that choices can be specified under choices or children.
        # I'm going to try to stick to just choices.
        # Aliases in the json format will make it more difficult
        # to use going forward.
        choices = list(kwargs_copy.pop(u"choices", [])) + \
            list(kwargs_copy.pop(u"children", []))
        Question.__init__(self, **kwargs_copy)
        for choice in choices:
            self.add_choice(**choice)

    def add_choice(self, **kwargs):
        option = Option(**kwargs)
        self.add_child(option)

    def validate(self):
        Question.validate(self)
        descendants = self.iter_descendants()
        next(descendants)  # iter_descendants includes self; we need to pop it

        for choice in descendants:
            choice.validate()

    def xml_control(self):
        assert self.bind[u"type"] in [u"select", u"select1"]
        survey = self.get_root()
        control_dict = self.control.copy()
        # Resolve field references in attributes
        for key, value in control_dict.items():
            control_dict[key] = survey.insert_xpaths(value)
        control_dict['ref'] = self.get_xpath()

        result = node(**control_dict)
        for element in self.xml_label_and_hint():
            result.appendChild(element)
        # itemset are only supposed to be strings,
        # check to prevent the rare dicts that show up
        if self['itemset'] and isinstance(self['itemset'], basestring):
            choice_filter = self.get('choice_filter')
            itemset, file_extension = os.path.splitext(self['itemset'])
            if file_extension in ['.csv', '.xml']:
                itemset = itemset
                itemset_label_ref = "label"
            else:
                itemset = self['itemset']
                itemset_label_ref = "jr:itext(itextId)"
            nodeset = "instance('" + itemset + "')/root/item"
            choice_filter = survey.insert_xpaths(choice_filter)
            if choice_filter:
                nodeset += '[' + choice_filter + ']'
            itemset_children = [node('value', ref='name'),
                                node('label', ref=itemset_label_ref)]
            result.appendChild(node('itemset', *itemset_children,
                                    nodeset=nodeset))
        else:
            for n in [o.xml() for o in self.children]:
                result.appendChild(n)
        return result


class SelectOneQuestion(MultipleChoiceQuestion):
    def __init__(self, **kwargs):
        super(SelectOneQuestion, self).__init__(**kwargs)
        self._dict[self.TYPE] = u"select one"


class Tag(SurveyElement):
    def __init__(self, **kwargs):
        kwargs_copy = kwargs.copy()
        choices = kwargs_copy.pop(u"choices", []) + \
            kwargs_copy.pop(u"children", [])

        super(Tag, self).__init__(**kwargs_copy)

        if choices:
            self.children = []

            for choice in choices:
                option = Option(**choice)
                self.add_child(option)

    def xml(self):
        result = node(u"tag", key=self.name)
        self.xml_label()
        result.appendChild(self.xml_label())
        for choice in self.children:
            result.appendChild(choice.xml())

        return result

    def validate(self):
        pass


class OsmUploadQuestion(UploadQuestion):
    def __init__(self, **kwargs):
        kwargs_copy = kwargs.copy()
        tags = kwargs_copy.pop(u"tags", []) + \
            kwargs_copy.pop(u"children", [])

        super(OsmUploadQuestion, self).__init__(**kwargs_copy)

        if tags:
            self.children = []

            for tag in tags:
                self.add_tag(**tag)

    def add_tag(self, **kwargs):
        tag = Tag(**kwargs)
        self.add_child(tag)

    def xml_control(self):
        control_dict = self.control
        control_dict['ref'] = self.get_xpath()
        control_dict['mediatype'] = self._get_media_type()
        result = node(
            u"upload",
            *self.xml_label_and_hint(),
            **control_dict
            )

        for osm_tag in self.children:
            result.appendChild(osm_tag.xml())

        return result


class RangeQuestion(Question):
    """
    This control string is the same for: strings, integers, decimals,
    dates, geopoints, barcodes ...
    """
    def xml_control(self):
        control_dict = self.control
        label_and_hint = self.xml_label_and_hint()
        survey = self.get_root()
        # Resolve field references in attributes
        for key, value in control_dict.items():
            control_dict[key] = survey.insert_xpaths(value)
        control_dict['ref'] = self.get_xpath()
        params = self.get('parameters', {})
        control_dict.update(params)
        result = node(**control_dict)
        if label_and_hint:
            for element in self.xml_label_and_hint():
                result.appendChild(element)

        return result
